@@ -1,11 +1,11 @@
 /*
  * misc.c	Various miscellaneous functions.
  *
- * Version:	$Id: misc.c,v 1.28 2002/03/13 16:48:39 aland Exp $
+ * Version:	$Id: misc.c,v 1.30 2002/06/21 19:57:25 cparker Exp $
  */
 
 static const char rcsid[] =
-"$Id: misc.c,v 1.28 2002/03/13 16:48:39 aland Exp $";
+"$Id: misc.c,v 1.30 2002/06/21 19:57:25 cparker Exp $";
 
 #include	"autoconf.h"
 
@@ -37,9 +37,11 @@ char * ip_hostname(char *buf, size_t buflen, uint32_t ipaddr)
 {
 	struct		hostent *hp;
 #ifdef GETHOSTBYADDRRSTYLE
+#if (GETHOSTBYADDRRSTYLE == SYSVSTYLE) || (GETHOSTBYADDRRSTYLE == GNUSTYLE)
 	char buffer[2048];
 	struct hostent result;
 	int error;
+#endif
 #endif
 
 	/*
@@ -163,7 +165,7 @@ void rad_lowercase(char *str) {
 	char *s;
 
 	for (s=str; *s; s++)
-		if (isupper(*s)) *s = tolower(*s);
+		if (isupper((int) *s)) *s = tolower((int) *s);
 }
 
 /*
@@ -174,7 +176,7 @@ void rad_rmspace(char *str) {
 	char *ptr = str;
 
   while(ptr && *ptr!='\0') {
-    while(isspace(*ptr))
+    while(isspace((int) *ptr))
       ptr++;
     *s = *ptr;
     ptr++;
