@@ -1,11 +1,11 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.69 2002/02/20 16:19:13 aland Exp $
+ * Version:	$Id: radius.c,v 1.77 2002/06/24 09:41:14 fcusack Exp $
  *
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.69 2002/02/20 16:19:13 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.77 2002/06/24 09:41:14 fcusack Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -42,10 +42,8 @@ static const char rcsid[] = "$Id: radius.c,v 1.69 2002/02/20 16:19:13 aland Exp
 
 /*
  *  The RFC says 4096 octets max, and most packets are less than 256.
- *  However, this number is just larger than the maximum MTU of just
- *  most types of networks, except maybe for gigabit ethernet.
  */
-#define PACKET_DATA_LEN 1600
+#define MAX_PACKET_LEN 4096
 
 /*
  *	The maximum number of attributes which we allow in an incoming
@@ -124,7 +122,7 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 		  u_short		total_length;
 		  int			len, allowed;
 		  int			msg_auth_offset = 0;
-		  uint8_t		data[4096];
+		  uint8_t		data[MAX_PACKET_LEN];
 		  
 		  /*
 		   *	Use memory on the stack, until we know how
@@ -314,16 +312,29 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 				  len = reply->length;
 
 				  /*
-				   *    Set the TAG at the beginning of the string if tagged.
-				   *    If tag value is not valid for tagged attribute, make 
-				   *    it 0x00 per RFC 2868.  -cparker
+				   *    Set the TAG at the beginning
+				   *    of the string if tagged.  If
+				   *    tag value is not valid for
+				   *    tagged attribute, make it 0x00
+				   *    per RFC 2868.  -cparker
 				   */
-				  if(reply->flags.has_tag) {
-				          len++;
-					  if(TAG_VALID(reply->flags.tag)) {
-					          *ptr++ = reply->flags.tag;
-					  }
-				  }
+				  if (reply->flags.has_tag) {
+					  if (TAG_VALID(reply->flags.tag)) {
+						  len++;
+						  *ptr++ = reply->flags.tag;
+
+					  } else if (reply->flags.encrypt == 2) {
+						  /*
+						   *  Tunnel passwords
+						   *  REQUIRE a tag,
+						   *  even if we don't
+						   *  have a valid
+						   *  tag.
+						   */
+						  len++;
+						  *ptr++ = 0x00;
+					  } /* else don't write a tag */
+				  } /* else the attribute doesn't have a tag */
 				 
 				  /*
 				   *	Ensure we don't go too far.
@@ -498,8 +509,9 @@ int rad_send(RADIUS_PACKET *packet, const RADIUS_PACKET *original, const char *s
 		   *	the VP list again only for debugging.
 		   */
 	} else if (librad_debug) {
-	  	DEBUG("Sending %s of id %d to %s\n", what, packet->id,
-		      ip_ntoa((char *)ip_buffer, packet->dst_ipaddr));
+	  	DEBUG("Re-sending %s of id %d to %s:%d\n", what, packet->id,
+		      ip_ntoa((char *)ip_buffer, packet->dst_ipaddr),
+		      packet->dst_port);
 		
 		for (reply = packet->vps; reply; reply = reply->next) {
 			/* FIXME: ignore attributes > 0xff */
@@ -617,7 +629,7 @@ RADIUS_PACKET *rad_recv(int fd)
 	radius_packet_t		*hdr;
 	char			host_ipaddr[16];
 	int			seen_eap;
-	uint8_t			data[4096];
+	uint8_t			data[MAX_PACKET_LEN];
 	int			num_attributes;
 	uint32_t                vendorcode;
 	int			vendorlen;
@@ -664,10 +676,28 @@ RADIUS_PACKET *rad_recv(int fd)
 
 	/*
 	 *	Check for packets smaller than the packet header.
+	 *
+	 *	RFC 2865, Section 3., subsection 'length' says:
+	 *
+	 *	"The minimum length is 20 ..."
 	 */
 	if (packet->data_len < AUTH_HDR_LEN) {
-		librad_log("WARNING: Malformed RADIUS packet from host %s: too short",
-			   ip_ntoa(host_ipaddr, packet->src_ipaddr));
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too short (%d < %d)",
+			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+			   packet->data_len, AUTH_HDR_LEN);
+		free(packet);
+		return NULL;
+	}
+
+	/*
+	 *	RFC 2865, Section 3., subsection 'length' says:
+	 *
+	 *	" ... and maximum length is 4096."
+	 */
+	if (packet->data_len > MAX_PACKET_LEN) {
+		librad_log("WARNING: Malformed RADIUS packet from host %s: too long (%d > %d)",
+			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
+			   packet->data_len, MAX_PACKET_LEN);
 		free(packet);
 		return NULL;
 	}
@@ -680,7 +710,16 @@ RADIUS_PACKET *rad_recv(int fd)
 	hdr = (radius_packet_t *)data;
 	memcpy(&len, hdr->length, sizeof(u_short));
 	totallen = ntohs(len);
-	if (packet->data_len != totallen) {
+
+	/*
+	 *	RFC 2865, Section 3., subsection 'length' says:
+	 *
+	 *	"If the packet is shorter than the Length field
+	 *	indicates, it MUST be silently discarded."
+	 *
+	 *	i.e. No response to the NAS.
+	 */
+	if (packet->data_len < totallen) {
 		librad_log("WARNING: Malformed RADIUS packet from host %s: received %d octets, packet size says %d",
 			   ip_ntoa(host_ipaddr, packet->src_ipaddr),
 			   packet->data_len, totallen);
@@ -688,6 +727,23 @@ RADIUS_PACKET *rad_recv(int fd)
 		return NULL;
 	}
 
+	/*
+	 *	RFC 2865, Section 3., subsection 'length' says:
+	 *
+	 *	"Octets outside the range of the Length field MUST be
+	 *	treated as padding and ignored on reception."
+	 */
+	if (packet->data_len > totallen) {
+		/*
+		 *	We're shortening the packet below, but just
+		 *	to be paranoid, zero out the extra data.
+		 */
+		memset(packet->data + totallen, 0,
+		       packet->data_len - totallen);
+
+		packet->data_len = totallen;
+	}
+
 	/*
 	 *	Walk through the packet's attributes, ensuring that
 	 *	they add up EXACTLY to the size of the packet.
@@ -1095,9 +1151,9 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			} else if (pair->flags.has_tag &&
 				   pair->type == PW_TYPE_STRING) {
 				if(TAG_VALID(*ptr)) {
-				       pair->flags.tag = *ptr++;
+					pair->flags.tag = *ptr++;
 					pair->length--;
-				} else if(pair->flags.encrypt == 2) {
+				} else if (pair->flags.encrypt == 2) {
 					/*
 					 * from RFC2868 - 3.5.  Tunnel-Password
 					 * If the value of the Tag field is greater than
@@ -1163,7 +1219,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original, const char *secre
 			if (pair->flags.has_tag &&
 			    pair->type == PW_TYPE_INTEGER) {
 			        pair->flags.tag = (pair->lvalue >> 24) & 0xff;
-				pair->lvalue &= 0xffffff;
+				pair->lvalue &= 0x00ffffff;
 			}
 			break;
 			
@@ -1315,6 +1371,7 @@ int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector
 	return pwlen;
 }
 
+
 /*
  *	Encode Tunnel-Password attributes when sending them out on the wire.
  *
@@ -1325,42 +1382,52 @@ int rad_pwdecode(char *passwd, int pwlen, const char *secret, const char *vector
  *      value MD5 hash.
  */
 
+static int saltoffset = 0; /* used to make all salts in a packet unique */
+
 int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char *vector)
 {
 	uint8_t	buffer[AUTH_VECTOR_LEN + MAX_STRING_LEN + 3];
 	unsigned char	digest[AUTH_VECTOR_LEN];
-	char    salt[2];
+	char*   salt;
 	int	i, n, secretlen;
-	int	len;
+	unsigned len;
+	
+
 	
 	len = *pwlen;
 
-	if (len < 3) {
-	  return 0;
-	}
-	salt[0] = passwd[0];
-	salt[1] = passwd[1];
-
-	/* Advance pointer past the salt, which is first two chars of passwd */
-	passwd = passwd + 2;
-	len -= 2;
+	if (len > 127) len = 127;
+	/*
+	 * Shift the password 3 positions right to place a salt and original
+	 * length, tag will be added automatically on packet send
+	 */
+	for (n=len ; n>=0 ; n--) passwd[n+3] = passwd[n];
+	salt = passwd;
+	passwd += 2;
+	/*
+	 * save original password length as first password character;
+	 */
 	*passwd = len;
+	len += 1;
+
+
+	/* generate salt */
+	saltoffset++;
+	salt[0] = (vector[0] ^ vector[1] ^ 0x3A ^ (char) ( (saltoffset>>8) & 0x000000ff ) ) | 0x80;
+	salt[1] = (vector[2] ^ vector[3] ^ vector[4] ^ (char) (saltoffset & 0x000000ff) );
 	
 	/*
 	 *	Padd password to multiple of AUTH_PASS_LEN bytes.
 	 */
-	if (len > 128) len = 128;
 	n = len % AUTH_PASS_LEN;
 	if (n) {
 		n = AUTH_PASS_LEN - n;
-		for (i = len; n > 0; n--, i++)
-			passwd[i] = 0;
-		len = i;
+		for (; n > 0; n--, len++)
+			passwd[len] = 0;
 	}
+	/* set new password length */
+	*pwlen = len + 2;
 	
-	*pwlen = len+2;
-	
-
 	/*
 	 *	Use the secret to setup the decryption digest
 	 */
@@ -1381,7 +1448,7 @@ int rad_tunnel_pwencode(char *passwd, int *pwlen, const char *secret, const char
 		for (i = 0; i < AUTH_PASS_LEN; i++)
 			passwd[i + n] ^= digest[i];
 	}
-
+	passwd[n] = 0;
 	return 0;
 }
 
@@ -1398,7 +1465,7 @@ int rad_tunnel_pwdecode(char *passwd, int * pwlen, const char *secret, const cha
 	unsigned char	digest[AUTH_VECTOR_LEN];
 	char    salt[2];
 	int	i, n, ntimes, secretlen;
-	int 	len;
+	unsigned len;
 	
 	len = *pwlen;
 
@@ -1425,7 +1492,7 @@ int rad_tunnel_pwdecode(char *passwd, int * pwlen, const char *secret, const cha
 			librad_md5_calc(digest, buffer, secretlen + AUTH_VECTOR_LEN + 2);
 		}
 		else {
-			memcpy(buffer + secretlen, passwd - AUTH_PASS_LEN, AUTH_PASS_LEN);
+			memcpy(buffer + secretlen, passwd + AUTH_PASS_LEN * (ntimes - 1), AUTH_PASS_LEN);
 			librad_md5_calc(digest, buffer, secretlen + AUTH_PASS_LEN);
 		}
 		for ( i = 0, n = ntimes * AUTH_PASS_LEN; i < AUTH_PASS_LEN && (i + n) < len; i++)
@@ -1433,13 +1500,16 @@ int rad_tunnel_pwdecode(char *passwd, int * pwlen, const char *secret, const cha
 	} while(ntimes--);
 	passwd[len] = '\0';
 
-	if (*(unsigned char*)passwd > len) {
+	if (*(unsigned char*)passwd >= len) {
 		/* Pasword is broken, original password should be longer */
 		*pwlen = 2;
 		passwd[0]=passwd[1]=0;
 		return 0;
 	}
-	return *pwlen = *passwd + 2; /* restore original length */
+	len = *pwlen = *passwd; /* restore original length */
+	for (n=0; n<len; n++) passwd[n-2]=passwd[n+1];
+	passwd[len-2] = 0;
+	return len;
 }
 
 /*
