@@ -2,7 +2,7 @@
  * token.c	Read the next token from a string.
  *		Yes it's pretty primitive but effective.
  *
- * Version:	$Id: token.c,v 1.12 2001/08/28 20:01:47 aland Exp $
+ * Version:	$Id: token.c,v 1.14 2002/06/21 19:57:25 cparker Exp $
  *
  */
 
@@ -12,7 +12,7 @@
 #include <ctype.h>
 #include "token.h"
 
-static const char rcsid[] = "$Id: token.c,v 1.12 2001/08/28 20:01:47 aland Exp $";
+static const char rcsid[] = "$Id: token.c,v 1.14 2002/06/21 19:57:25 cparker Exp $";
 
 typedef struct {
 	const char *str;
@@ -30,6 +30,8 @@ static const TOKEN tokens[] = {
 	{ "+=",	T_OP_ADD,	},
 	{ "-=",	T_OP_SUB,	},
 	{ ":=",	T_OP_SET,	},
+	{ "=*", T_OP_CMP_TRUE,  },
+	{ "!*", T_OP_CMP_FALSE, },
 	{ "==",	T_OP_CMP_EQ,	},
 	{ "=",	T_OP_EQ,	},
 	{ "!=",	T_OP_NE,	},
@@ -70,7 +72,7 @@ static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
 
 	/* Skip whitespace */
 	p = *ptr;
-	while (*p && isspace(*p))
+	while (*p && isspace((int) *p))
 		p++;
 
 	if (*p == 0) {
@@ -85,7 +87,7 @@ static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
 		if (TOKEN_MATCH(p, t->str)) {
 			strcpy(buf, t->str);
 			p += strlen(t->str);
-			while (isspace(*p))
+			while (isspace((int) *p))
 				p++;
 			*ptr = p;
 			return t->token;
@@ -139,7 +141,7 @@ static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
 			break;
 		}
 		if (!quote) {
-			if (isspace(*p))
+			if (isspace((int) *p))
 				break;
 			if (tok) {
 				for (t = tokenlist; t->str; t++)
@@ -154,7 +156,7 @@ static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
 	*s++ = 0;
 
 	/* Skip whitespace again. */
-	while (*p && isspace(*p))
+	while (*p && isspace((int) *p))
 		p++;
 	*ptr = p;
 
