@@ -1,11 +1,11 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.47 2002/02/01 16:17:08 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.50 2002/06/21 19:57:25 cparker Exp $
  *
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.47 2002/02/01 16:17:08 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.50 2002/06/21 19:57:25 cparker Exp $";
 
 #include	"autoconf.h"
 
@@ -458,7 +458,7 @@ static int gettime(const char *valstr, time_t *lvalue)
 
 	strNcpy(buf, valstr, sizeof(buf));
 	for (p = buf; *p; p++)
-		if (isupper(*p)) *p = tolower(*p);
+		if (isupper((int) *p)) *p = tolower((int) *p);
 
 	p = buf;
 	day = mystrtok(&p, " \t");
@@ -516,12 +516,15 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 		         /* Wildcard tag for check items */
 		         tag = TAG_ANY;
 			 *ts = 0;
-		 } else {
+		 } else if ((ts[1] >= 0) && (ts[1] <= '9')) {
 		         /* It's not a wild card tag */
 		         tag = strtol(ts + 1, &tc, 0);
 			 if (tc && !*tc && TAG_VALID_ZERO(tag))
-			        *ts = 0;
+				 *ts = 0;
 			 else tag = 0;
+		 } else {
+			 librad_log("Invalid tag for attribute %s", attribute);
+			 return NULL;
 		 }
 		 found_tag = 1;
 	}
@@ -575,12 +578,26 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 		found_tag = 1;
 	}	
 	
+	if(found_tag) {
+	  vp->flags.tag = tag;
+	}
+
 	/*
 	 *	Even for integers, dates and ip addresses we
 	 *	keep the original string in vp->strvalue.
 	 */
 	strNcpy((char *)vp->strvalue, value, MAX_STRING_LEN);
 
+	/*
+	 *      For =* and !* operators, the value is irrelevant
+	 *      so we return now.
+	 */
+
+	if( vp->operator == T_OP_CMP_TRUE 
+	    || vp->operator == T_OP_CMP_FALSE ) {
+	        return vp;
+	}
+
 	switch(da->type) {
 		case PW_TYPE_STRING:
 			vp->length = strlen(value);
@@ -614,7 +631,7 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 			 *
 			 *	Note that ALL integers are unsigned!
 			 */
-			if (isdigit(*value)) {
+			if (isdigit((int) *value)) {
 				vp->lvalue = atoi(value);
 				vp->length = 4;
 			}
@@ -690,9 +707,6 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 			librad_log("unknown attribute type %d", da->type);
 			return NULL;
 	}
-	if(found_tag) {
-	  vp->flags.tag = tag;
-	}
 	return vp;
 }
 
