@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.101 2002/03/06 17:49:36 aland Exp $
+ * Version:	$Id: auth.c,v 1.106 2002/06/21 19:57:25 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.101 2002/03/06 17:49:36 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.106 2002/06/21 19:57:25 cparker Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -56,8 +56,8 @@ char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli) {
 	if ((pair = pairfind(request->packet->vps, PW_NAS_PORT_ID)) != NULL)
 		port = pair->lvalue;
 
-	snprintf(buf, buflen, "from nas %.128s port %d%s%.128s",
-			nas_name2(request->packet), port,
+	snprintf(buf, buflen, "from client %.128s port %d%s%.128s",
+			client_name(request->packet->src_ipaddr), port,
 			(do_cli ? " cli " : ""), (do_cli ? (char *)cli->strvalue : ""));
 
 	return buf;
@@ -211,6 +211,7 @@ int rad_check_password(REQUEST *request)
 	VALUE_PAIR *password_pair;
 	VALUE_PAIR *auth_item;
 	char string[MAX_STRING_LEN];
+	const char *crypted_password;
 	int auth_type = -1;
 	int result;
 	int auth_type_count = 0;
@@ -296,10 +297,17 @@ int rad_check_password(REQUEST *request)
 				return -1;
 			}
 					
+			crypted_password = crypt((char *)auth_item->strvalue,
+						 (char *)password_pair->strvalue);
+			if (!crypted_password) {
+				rad_authlog("Login incorrect "
+					    "(system failed to supply an encrypted password for comparison)", request, 0);
+				return -1;
+			}
 			if (strcmp((char *)password_pair->strvalue,
-				   crypt((char *)auth_item->strvalue,
-					 (char *)password_pair->strvalue)) != 0)
-				result = -1;
+				   crypted_password) != 0) {
+				return -1;
+			}
 			break;
 		case PW_AUTHTYPE_LOCAL:
 			DEBUG2("auth: type Local");
@@ -405,6 +413,7 @@ int rad_authenticate(REQUEST *request)
 	VALUE_PAIR	*check_item;
 	VALUE_PAIR	*reply_item;
 	VALUE_PAIR	*auth_item;
+	VALUE_PAIR	*module_msg;
 	VALUE_PAIR	*tmp = NULL;
 	VALUE_PAIR	*autz_type_item = NULL;
 	int		result, r;
@@ -458,9 +467,14 @@ int rad_authenticate(REQUEST *request)
 		/*
 		 *	Initialize our reply to the user, by taking
 		 *	the reply attributes from the proxy.
+		 *
+		 *	Note that we DELETE the Proxy-State attributes
+		 *	from the proxy reply, as they include the one
+		 *	we added, which MUST NOT go back to the NAS.
 		 */
 		if (request->proxy_reply->vps) {
 			request->reply->vps = request->proxy_reply->vps;
+			pairdelete(&request->reply->vps, PW_PROXY_STATE);
 			request->reply->code = request->proxy_reply->code;
 			request->proxy_reply->vps = NULL;
 		}
@@ -521,10 +535,8 @@ autz_redo:
 			r != RLM_MODULE_OK &&
 			r != RLM_MODULE_UPDATED) {
 		if (r != RLM_MODULE_FAIL && r != RLM_MODULE_HANDLED) {
-			VALUE_PAIR *module_msg;
-
 			if ((module_msg = pairfind(request->packet->vps,
-					PW_MODULE_MESSAGE)) != NULL){
+					PW_MODULE_FAILURE_MESSAGE)) != NULL){
 				char msg[MAX_STRING_LEN+16];
 				snprintf(msg, sizeof(msg), "Invalid user (%s)",
 					 module_msg->strvalue);
@@ -588,12 +600,10 @@ autz_redo:
 	 *	wants to send back.
 	 */
 	if (result < 0) {
-		VALUE_PAIR *module_msg;
-
 		DEBUG2("auth: Failed to validate the user.");
 		request->reply->code = PW_AUTHENTICATION_REJECT;
-		
-		if ((module_msg = pairfind(request->packet->vps,PW_MODULE_MESSAGE)) != NULL){
+
+		if ((module_msg = pairfind(request->packet->vps,PW_MODULE_FAILURE_MESSAGE)) != NULL){
 			char msg[MAX_STRING_LEN+19];
 
 			snprintf(msg, sizeof(msg), "Login incorrect (%s)",
@@ -610,7 +620,7 @@ autz_redo:
 
 			p = auth_item->strvalue;
 			while (*p != '\0') {
-				if (!isprint(*p)) {
+				if (!isprint((int) *p)) {
 					log_debug("  WARNING: Unprintable characters in the password.\n\t  Double-check the shared secret on the server and the NAS!");
 					break;
 				}
@@ -869,7 +879,16 @@ autz_redo:
 	if (request->reply->code == 0)
 	  request->reply->code = PW_AUTHENTICATION_ACK;
 
-	rad_authlog("Login OK", request, 1);
+	if ((module_msg = pairfind(request->packet->vps,PW_MODULE_SUCCESS_MESSAGE)) != NULL){
+		char msg[MAX_STRING_LEN+12];
+
+		snprintf(msg, sizeof(msg), "Login OK (%s)",
+			 module_msg->strvalue);
+		rad_authlog(msg, request, 1);
+	} else {
+		rad_authlog("Login OK", request, 1);
+	}
+
 	if (exec_program && !exec_wait) {
 		/*
 		 *	No need to check the exit status here.
