@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.80 2002/02/26 21:46:17 aland Exp $
+ * Version:	$Id: conffile.c,v 1.84 2002/05/29 16:18:35 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.80 2002/02/26 21:46:17 aland Exp $";
+"$Id: conffile.c,v 1.84 2002/05/29 16:18:35 aland Exp $";
 
 #define xstrdup strdup
 
@@ -239,7 +239,7 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 {
 	char *p;
 	const char *end, *ptr;
-	char name[1024];
+	char name[8192];
 	CONF_PAIR *cpn;
 	CONF_SECTION *outercs;
 
@@ -353,7 +353,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base, const CONF_PARSER *variables)
 	CONF_PAIR *cp;
 	CONF_SECTION *subsection;
 	uint32_t ipaddr;
-	char buffer[1024];
+	char buffer[8192];
 	const char *value;
 	void *data;
 
@@ -492,9 +492,9 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 	char *ptr;
 	const char *value;
 	char buf[8192];
-	char buf1[1024];
-	char buf2[1024];
-	char buf3[1024];
+	char buf1[8192];
+	char buf2[8192];
+	char buf3[8192];
 	int t1, t2, t3;
 	
 	/*
@@ -640,7 +640,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 		if (buf1[0] != 0 && buf2[0] == 0 && buf3[0] == 0) {
 			t2 = T_OP_EQ;
 		} else if (buf1[0] == 0 || buf2[0] == 0 || 
-					(t2 < T_EQSTART || t2 > T_EQEND)) {
+			   (t2 < T_EQSTART || t2 > T_EQEND)) {
 			radlog(L_ERR, "%s[%d]: Line is not in 'attribute = value' format",
 					cf, *lineno);
 			cf_section_free(&cs);
@@ -657,7 +657,7 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 			cf_section_free(&cs);
 			return NULL;
 		}
-		
+
 		/*
 		 *	Handle variable substitution via ${foo}
 		 */
@@ -720,6 +720,75 @@ static CONF_SECTION *conf_read(const char *fromfile, int fromline,
 	return cs;
 }
 
+/*
+ *	Xlat for %{config:section.subsection.attribute}
+ */
+static int xlat_config(void *instance, REQUEST *request,
+		       char *fmt, char *out, int outlen,
+		       RADIUS_ESCAPE_STRING func)
+{
+	CONF_SECTION *cs;
+	CONF_PAIR *cp;
+	char buffer[1024];
+	char *p;
+	const char *start = fmt;
+
+	cp = NULL;
+	cs = NULL;
+
+	while (cp == NULL) {
+		/*
+		 *	Find the next section.
+		 */
+		for (p = buffer; (*fmt != 0) && (*fmt != '.'); p++, fmt++) {
+			*p = *fmt;
+		}
+		*p = '\0';
+
+		/*
+		 *  The character is a '.', find a section (as the user
+		 *  has given us a subsection to find)
+		 */
+		if (*fmt == '.') {
+			CONF_SECTION *next;
+
+			fmt++;	/* skip the period */
+
+			if (cs == NULL) {
+			  next = cf_section_find(buffer);
+			} else {
+			  next = cf_subsection_find_next(cs, NULL, buffer);
+			}
+			if (next == NULL) {
+				radlog(L_ERR, "config: No such section %s in format string %s", buffer, start);
+				return 0;
+			}
+			cs = next;
+
+		} else {	/* no period, must be a conf-part */
+			cp = cf_pair_find(cs, buffer);
+
+			if (cp == NULL) {
+				radlog(L_ERR, "config: No such section %s in format string %s", buffer, start);
+				return 0;
+			}
+		}
+	} /* until cp is non-NULL */
+
+	/*
+	 *  Ensure that we only copy what's necessary.
+	 *
+	 *  If 'outlen' is too small, then the output is chopped to fit.
+	 */
+	if (outlen > strlen(cp->value)) {
+		outlen = strlen(cp->value) + 1;
+	}
+	
+	strNcpy(out, cp->value, outlen);
+    
+	return outlen;
+}
+
 /*
  *	These are not used anywhere else..
  */
@@ -829,6 +898,11 @@ int read_radius_conf_file(void)
 		return -1;
 	}
 
+	/*
+	 *  Register the %{config:section.subsection} xlat function.
+	 */
+	xlat_register("config", xlat_config, NULL);
+
 	return 0;	
 }
 
@@ -963,6 +1037,7 @@ static int generate_realms(const char *filename)
 		if ((cf_section_value_find(cs, "notsuffix")) != NULL)
 			c->notrealm = 1;
 		c->active = TRUE;
+		c->acct_active = TRUE;
 
 		c->next = NULL;
 		*tail = c;
@@ -988,6 +1063,7 @@ static int generate_clients(const char *filename)
 	CONF_SECTION	*cs;
 	RADCLIENT	*c;
 	char		*hostnm, *secret, *shortnm, *netmask;
+	char            *nastype, *login, *password;
 
 	for (cs = cf_subsection_find_next(config, NULL, "client"); cs != NULL; 
 			cs = cf_subsection_find_next(config, cs, "client")) {
@@ -1026,10 +1102,40 @@ static int generate_clients(const char *filename)
 					strlen(shortnm), sizeof(c->shortname) - 1);
 			return -1;
 		}
+
+		if((nastype = cf_section_value_find(cs, "nastype")) != NULL) {
+		        if(strlen(nastype) >= sizeof(c->nastype)) { 
+			       radlog(L_ERR, "%s[%d]: nastype of length %d longer than the allowed maximum of %d",
+				      filename, cs->item.lineno, 
+				      strlen(nastype), sizeof(c->nastype) - 1);
+			       return -1;
+			}
+		}
+
+		if((login = cf_section_value_find(cs, "login")) != NULL) {
+		        if(strlen(login) >= sizeof(c->login)) { 
+			       radlog(L_ERR, "%s[%d]: login of length %d longer than the allowed maximum of %d",
+				      filename, cs->item.lineno, 
+				      strlen(login), sizeof(c->login) - 1);
+			       return -1;
+			}
+		}
+
+		if((password = cf_section_value_find(cs, "password")) != NULL) {
+		        if(strlen(password) >= sizeof(c->password)) { 
+			       radlog(L_ERR, "%s[%d]: password of length %d longer than the allowed maximum of %d",
+				      filename, cs->item.lineno, 
+				      strlen(password), sizeof(c->password) - 1);
+			       return -1;
+			}
+		}
+
 		/*
 		 * The size is fine.. Let's create the buffer
 		 */
 		c = rad_malloc(sizeof(RADCLIENT));
+		memset(c, 0, sizeof(RADCLIENT));
+
 
 		/*
 		 *	Look for netmasks.
@@ -1075,6 +1181,12 @@ static int generate_clients(const char *filename)
 
 		strcpy((char *)c->secret, secret);
 		strcpy(c->shortname, shortnm);
+		if(nastype != NULL)
+		        strcpy(c->nastype, nastype);
+		if(login != NULL)
+		        strcpy(c->login, login);
+		if(password != NULL)
+		        strcpy(c->password, password);
 
 		c->next = clients;
 		clients = c;
