@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.63 2002/03/02 16:13:49 kkalev Exp $
+ * Version:	$Id: modules.c,v 1.66 2002/06/14 15:17:38 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-static const char rcsid[] = "$Id: modules.c,v 1.63 2002/03/02 16:13:49 kkalev Exp $";
+static const char rcsid[] = "$Id: modules.c,v 1.66 2002/06/14 15:17:38 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -71,7 +71,10 @@ const char *component_names[RLM_COMPONENT_COUNT] =
 	"authorize",
 	"preacct",
 	"accounting",
-	"session"
+	"session",
+	"pre-proxy",
+	"post-proxy",
+	"post-auth"
 };
 
 static const char *subcomponent_names[RLM_COMPONENT_COUNT] =
@@ -80,7 +83,10 @@ static const char *subcomponent_names[RLM_COMPONENT_COUNT] =
 	"autztype",
 	"preacctype",
 	"acctype",
-	"sesstype"
+	"sesstype",
+	"pre-proxytype",
+	"post-proxytype",
+	"post-authtype"
 };
 
 static void indexed_modcallable_free(indexed_modcallable **cf)
@@ -197,7 +203,9 @@ static module_list_t *linkto_module(const char *module_name,
 		const char *cffilename, int cflineno)
 {
 	module_list_t **last, *node;
-	lt_dlhandle *handle;
+	lt_dlhandle handle;
+	char module_struct[256];
+	char *p;
 
 	/*
 	 *	Look through the global module library list for the
@@ -238,7 +246,12 @@ static module_list_t *linkto_module(const char *module_name,
 	/*
 	 *	Link to the module's rlm_FOO{} module structure.
 	 */
-	node->module = (module_t *) lt_dlsym(node->handle, module_name);
+	/* module_name has the version embedded; strip it. */
+	strcpy(module_struct, module_name);
+	p = strrchr(module_struct, '-');
+	if (p)
+		*p = '\0';
+	node->module = (module_t *) lt_dlsym(node->handle, module_struct);
 	if (!node->module) {
 		radlog(L_ERR|L_CONS, "%s[%d] Failed linking to "
 				"%s structure in %s: %s\n",
@@ -336,11 +349,24 @@ module_instance_t *find_module_instance(const char *instname)
 	node->insthandle = NULL;
 	
 	/*
-	 *	Link to the module by name: rlm_FOO
+	 *	Link to the module by name: rlm_FOO-major.minor
 	 */
-	snprintf(module_name, sizeof(module_name), "rlm_%s", name1);
-	node->entry = linkto_module(module_name,
-			"radiusd.conf", cf_section_lineno(inst_cs));
+	if (strncmp(name1, "rlm_", 4)) {
+#if 0
+		snprintf(module_name, sizeof(module_name), "rlm_%s-%d.%d",
+			 name1, RADIUSD_MAJOR_VERSION, RADIUSD_MINOR_VERSION);
+#else
+		snprintf(module_name, sizeof(module_name), "rlm_%s",
+			 name1);
+#endif
+	} else {
+		strNcpy(module_name, name1, sizeof(module_name));
+
+	}
+
+	/* XXX "radiusd.conf" is wrong here; must find cf filename */
+	node->entry = linkto_module(module_name, "radiusd.conf",
+				    cf_section_lineno(inst_cs));
 	if (!node->entry) {
 		free(node);
 		/* linkto_module logs any errors */
@@ -450,6 +476,9 @@ static int indexed_modcall(int comp, int idx, REQUEST *request)
 			case RLM_COMPONENT_PREACCT: return RLM_MODULE_NOOP;
 			case RLM_COMPONENT_ACCT:    return RLM_MODULE_NOOP;
 			case RLM_COMPONENT_SESS:    return RLM_MODULE_FAIL;
+			case RLM_COMPONENT_PRE_PROXY:  return RLM_MODULE_NOOP;
+			case RLM_COMPONENT_POST_PROXY: return RLM_MODULE_NOOP;
+			case RLM_COMPONENT_POST_AUTH:  return RLM_MODULE_NOOP;
 			default:                    return RLM_MODULE_FAIL;
 		}
 	}
@@ -685,3 +714,28 @@ int module_checksimul(REQUEST *request, int maxsimul)
 
 	return (request->simul_count < maxsimul) ? 0 : request->simul_mpp;
 }
+
+/*
+ *	Do pre-proxying for ALL configured sessions
+ */
+int module_pre_proxy(REQUEST *request)
+{
+	return indexed_modcall(RLM_COMPONENT_PRE_PROXY, 0, request);
+}
+
+/*
+ *	Do post-proxying for ALL configured sessions
+ */
+int module_post_proxy(REQUEST *request)
+{
+	return indexed_modcall(RLM_COMPONENT_POST_PROXY, 0, request);
+}
+
+/*
+ *	Do post-authentication for ALL configured sessions
+ */
+int module_post_auth(REQUEST *request)
+{
+	return indexed_modcall(RLM_COMPONENT_POST_AUTH, 0, request);
+}
+
