@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.242 2002/03/08 16:35:26 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.253 2002/06/25 20:45:13 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.242 2002/03/08 16:35:26 aland Exp $";
+"$Id: radiusd.c,v 1.253 2002/06/25 20:45:13 cparker Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -197,8 +197,8 @@ static CONF_PARSER server_config[] = {
 	{ "log_auth_goodpass", PW_TYPE_BOOLEAN, 0, &mainconfig.log_auth_goodpass, "no" },
 	{ "pidfile", PW_TYPE_STRING_PTR, 0, &pid_file, "${run_dir}/radiusd.pid"},
 	{ "bind_address", PW_TYPE_IPADDR, 0, &myip, "*" },
-	{ "user", PW_TYPE_STRING_PTR, 0, &uid_name, "nobody"},
-	{ "group", PW_TYPE_STRING_PTR, 0, &gid_name, "nobody"},
+	{ "user", PW_TYPE_STRING_PTR, 0, &uid_name, NULL},
+	{ "group", PW_TYPE_STRING_PTR, 0, &gid_name, NULL},
 	{ "usercollide", PW_TYPE_BOOLEAN, 0, &mainconfig.do_usercollide,  "no" },
 	{ "lower_user", PW_TYPE_STRING_PTR, 0, &mainconfig.do_lower_user, "no" },
 	{ "lower_pass", PW_TYPE_STRING_PTR, 0, &mainconfig.do_lower_pass, "no" },
@@ -214,49 +214,46 @@ static CONF_PARSER server_config[] = {
 static int switch_users() {
 	
 	/*
-	 *  Set the UID and GID, but only if we're NOT running
-	 *  in debugging mode.
+	 *  Switch UID and GID to what is specified in the config file
 	 */
-	if (!debug_flag) {
 
-		/*  Set GID.  */
-		if (gid_name != NULL) {
-			struct group *gr;
+	/*  Set GID.  */
+	if (gid_name != NULL) {
+		struct group *gr;
 
-			gr = getgrnam(gid_name);
-			if (gr == NULL) {
-				if (errno == ENOMEM) {
-					radlog(L_ERR|L_CONS, "Cannot switch to Group %s: out of memory", gid_name);
-				} else {
-					radlog(L_ERR|L_CONS, "Cannot switch group; %s doesn't exist", gid_name);
-				}
-				exit(1);
-			}
-			server_gid = gr->gr_gid;
-			if (setgid(server_gid) < 0) {
-				radlog(L_ERR|L_CONS, "Failed setting Group to %s: %s", gid_name, strerror(errno));
-				exit(1);
+		gr = getgrnam(gid_name);
+		if (gr == NULL) {
+			if (errno == ENOMEM) {
+				radlog(L_ERR|L_CONS, "Cannot switch to Group %s: out of memory", gid_name);
+			} else {
+				radlog(L_ERR|L_CONS, "Cannot switch group; %s doesn't exist", gid_name);
 			}
+			exit(1);
 		}
+		server_gid = gr->gr_gid;
+		if (setgid(server_gid) < 0) {
+			radlog(L_ERR|L_CONS, "Failed setting Group to %s: %s", gid_name, strerror(errno));
+			exit(1);
+		}
+	}
 
-		/*  Set UID.  */
-		if (uid_name != NULL) {
-			struct passwd *pw;
+	/*  Set UID.  */
+	if (uid_name != NULL) {
+		struct passwd *pw;
 
-			pw = getpwnam(uid_name);
-			if (pw == NULL) {
-				if (errno == ENOMEM) {
-					radlog(L_ERR|L_CONS, "Cannot switch to User %s: out of memory", uid_name);
-				} else {
-					radlog(L_ERR|L_CONS, "Cannot switch user; %s doesn't exist", uid_name);
-				}
-				exit(1);
-			}
-			server_uid = pw->pw_uid;
-			if (setuid(server_uid) < 0) {
-				radlog(L_ERR|L_CONS, "Failed setting User to %s: %s", uid_name, strerror(errno));
-				exit(1);
+		pw = getpwnam(uid_name);
+		if (pw == NULL) {
+			if (errno == ENOMEM) {
+				radlog(L_ERR|L_CONS, "Cannot switch to User %s: out of memory", uid_name);
+			} else {
+				radlog(L_ERR|L_CONS, "Cannot switch user; %s doesn't exist", uid_name);
 			}
+			exit(1);
+		}
+		server_uid = pw->pw_uid;
+		if (setuid(server_uid) < 0) {
+			radlog(L_ERR|L_CONS, "Failed setting User to %s: %s", uid_name, strerror(errno));
+			exit(1);
 		}
 	}
 	return(0);
@@ -863,6 +860,7 @@ int main(int argc, char *argv[])
 	 */
 	radius_pid = getpid();
 
+
 	/*
 	 *  Only write the PID file if we're running as a daemon.
 	 *
@@ -1025,7 +1023,7 @@ int main(int argc, char *argv[])
 				
 			} else {    /* It came in on the proxy port */
 				REALM *rl;
-				if ((rl = realm_findbyaddr(packet->src_ipaddr)) == NULL) {
+				if ((rl = realm_findbyaddr(packet->src_ipaddr,packet->src_port)) == NULL) {
 					radlog(L_ERR, "Ignoring request from unknown proxy %s:%d",
 					ip_ntoa((char *)buffer, packet->src_ipaddr),
 					packet->src_port);
@@ -1043,7 +1041,9 @@ int main(int argc, char *argv[])
 			 *  packets here will make our life easier.
 			 */
 			if (packet->code > PW_ACCESS_CHALLENGE) {
-				radlog(L_ERR, "Ignoring request from client %s:%d with unknown code %d", buffer, packet->src_port, packet->code);
+				radlog(L_ERR, "Ignoring request from client %s:%d with unknown code %d",
+				       ip_ntoa((char *)buffer, packet->src_ipaddr),
+				       packet->src_port, packet->code);
 				rad_free(&packet);
 				continue;
 			}
@@ -1080,8 +1080,8 @@ int main(int argc, char *argv[])
 		 *  should be wrapped, and handled in a thread pool.
 		 */
 		if ((rad_snmp.smux_fd >= 0) &&
-				FD_ISSET(rad_snmp.smux_fd, &readfds) &&
-				(rad_snmp.smux_event == SMUX_READ)) {
+		    FD_ISSET(rad_snmp.smux_fd, &readfds) &&
+		    (rad_snmp.smux_event == SMUX_READ)) {
 			smux_read();
 		}
 		
@@ -1285,8 +1285,8 @@ static void rad_reject(REQUEST *request)
 		/*
 		 *  Accounting requests, etc. get dropped on the floor.
 		 */
-		case PW_ACCOUNTING_REQUEST:
 		default:
+		case PW_ACCOUNTING_REQUEST:
 			break;
 
 		/*
@@ -1436,7 +1436,6 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	RADIUS_PACKET *packet, *original;
 	const char *secret;
 	int finished = FALSE;
-	int proxy_sent = 0;
 	int reprocess = 0;
 	
 	/*
@@ -1523,30 +1522,33 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 
 	(*fun)(request);
 
-	/* See if we have to lower user/pass after processing */
-	if(strcmp(mainconfig.do_lower_user, "after") == 0) {
-		rad_lowerpair(request, request->username);
-		reprocess = 1;
-	}
-	if(strcmp(mainconfig.do_lower_pass, "after") == 0) {
+	/*
+	 *  Reprocess if we rejected last time
+	 */
+	if ((fun == rad_authenticate) &&
+	    (request->reply->code == PW_AUTHENTICATION_REJECT)) {
+	  /* See if we have to lower user/pass after processing */
+	  if (strcmp(mainconfig.do_lower_user, "after") == 0) {
+		  rad_lowerpair(request, request->username);
+		  reprocess = 1;
+	  }
+	  if (strcmp(mainconfig.do_lower_pass, "after") == 0) {
 		rad_lowerpair(request, rad_getpass(request));
 		reprocess = 1;
-	}
-	if(strcmp(mainconfig.do_nospace_user, "after") == 0) {
-		rad_rmspace_pair(request, request->username);
-		reprocess = 1;
-	}
-	if(strcmp(mainconfig.do_nospace_pass, "after") == 0) {
-		rad_rmspace_pair(request, rad_getpass(request));
-		reprocess = 1;
-	}
-
-	/* Reprocess if we rejected last time */
-	if ((fun == rad_authenticate) &&
-	    (request->reply->code == PW_AUTHENTICATION_REJECT) &&
-	    (reprocess))  {
-		pairfree(&request->config_items);
-		(*fun)(request);
+	  }
+	  if (strcmp(mainconfig.do_nospace_user, "after") == 0) {
+		  rad_rmspace_pair(request, request->username);
+		  reprocess = 1;
+	  }
+	  if (strcmp(mainconfig.do_nospace_pass, "after") == 0) {
+		  rad_rmspace_pair(request, rad_getpass(request));
+		  reprocess = 1;
+	  }
+	  
+	  if (reprocess) {
+		  pairfree(&request->config_items);
+		  (*fun)(request);
+	  }
 	}
 	
 	/*
@@ -1556,6 +1558,8 @@ int rad_respond(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 */
 	if (proxy_requests) {
 		if (request->proxy == NULL) {
+			int proxy_sent = 0;
+
 			/*
 			 *  Try to proxy this request.  Returns:
 			 *  -1: error, drop the request
@@ -1657,33 +1661,27 @@ finished_request:
 	 *
 	 *  Hmm... cleaning them up in the child thread also seems
 	 *  to make the server run more efficiently!
-	 */
-
-	/*
-	 *  If we proxied this request, it's not safe to delete it until
-	 *  after the proxy reply.
 	 *
-	 *  However, we DO mark the request as no longer being handled
-	 *  by a thread.
+	 *  If we've delayed the REJECT, then do NOT clean up the request,
+	 *  as we haven't created the REJECT message yet.
 	 */
-	if (proxy_sent) {
-		goto postpone_request;
+	if ((request->options & RAD_REQUEST_OPTION_DELAYED_REJECT) == 0) {
+		if (request->packet) {
+			pairfree(&request->packet->vps);
+			request->username = NULL;
+			request->password = NULL;
+		}
+
+		/*
+		 *  If we've sent a reply to the NAS, then this request is
+		 *  pretty much finished, and we have no more need for any
+		 *  of the value-pair's in it, including the proxy stuff.
+		 */
+		if (request->reply->code != 0) {
+			pairfree(&request->reply->vps);
+		}
 	}
 
-	if (request->packet) {
-		pairfree(&request->packet->vps);
-		request->username = NULL;
-		request->password = NULL;
-	}
-
-	/*
-	 *  If we've sent a reply to the NAS, then this request is
-	 *  pretty much finished, and we have no more need for any
-	 *  of the value-pair's in it, including the proxy stuff.
-	 */
-	if (request->reply->code != 0) {
-		pairfree(&request->reply->vps);
-	}
 	pairfree(&request->config_items);
 	if (request->proxy) {
 		pairfree(&request->proxy->vps);
@@ -1843,9 +1841,42 @@ static struct timeval *rad_clean_list(time_t now)
 	 *  Return NULL, so that the select() call will sleep forever.
 	 */
 	if (info.smallest < 0) {
-		DEBUG2("Nothing to do.  Sleeping until we see a request.");
-		last_tv_ptr = NULL;
-		return NULL;
+		/*
+		 *  If we're not proxying, then there really isn't anything
+		 *  to do.
+		 *
+		 *  If we ARE proxying, then we can safely sleep
+		 *  forever if we're told to NEVER send proxy retries
+		 *  ourselves, until the NAS kicks us again.
+		 * 
+		 *  Otherwise, there are no outstanding requests, then
+		 *  we can sleep forever.  This happens when we get
+		 *  woken up with a bad packet.  It's discarded, so if
+		 *  there are no live requests, we can safely sleep
+		 *  forever.
+		 */
+		if ((!proxy_requests) ||
+		    proxy_synchronous ||
+		    (rl_num_requests() == 0)) {
+			DEBUG2("Nothing to do.  Sleeping until we see a request.");
+			last_tv_ptr = NULL;
+			return NULL;
+		}
+
+		/*
+		 *  We ARE proxying.  In that case, we avoid a race condition
+		 *  where a child thread handling a request proxies the
+		 *  packet, and sets the retry delay.  In that case, we're
+		 *  supposed to wake up in N seconds, but we can't, as
+		 *  we're sleeping forever.
+		 *
+		 *  Instead, we prevent the problem by waking up anyhow
+		 *  at the 'proxy_retry_delay' time, even if there's
+		 *  nothing to do.  In the worst case, this will cause
+		 *  the server to wake up every N seconds, to do a small
+		 *  amount of unnecessary work.
+		 */
+		info.smallest = proxy_retry_delay;
 	}
 	/*
 	 *  Set the time (in seconds) for how long we're
@@ -1902,113 +1933,187 @@ static REQUEST *rad_check_list(REQUEST *request)
 	curreq = rl_find(request);
 	if (curreq != NULL) {
 		/*
-		 *  We now check the authentication vectors.
-		 *  If the client has sent us a request with
-		 *  identical code && ID, but different vector,
-		 *  then they MUST have gotten our response, so
-		 *  we can delete the original request, and process
-		 *  the new one.
+		 *	If the request (duplicate or now) is currently
+		 *	being processed, then discard the new request.
+		 */
+		if (curreq->child_pid != NO_SUCH_CHILD_PID) {
+			radlog(L_ERR, "Discarding new request from "
+			       "client %s:%d - ID: %d due to live request %d",
+			       client_name(curreq->packet->src_ipaddr),
+			       curreq->packet->src_port, curreq->packet->id,
+			       curreq->number);
+			request_free(&request);
+			return NULL;
+		}
+
+		/*
+		 *	The current request isn't finished, which
+		 *	means that the NAS sent us a new packet, while
+		 *	we were waiting for a proxy response.
 		 *
-		 *  If the vectors are the same, then it's a duplicate
-		 *  request, and we can send a duplicate reply.
+		 *	In that case, it doesn't matter if the vectors
+		 *	are the same, as deleting the un-finished request
+		 *	would mean that the (eventual) proxy response would
+		 *	be associated with the wrong NAS request.
+		 */
+		if (!curreq->finished) {
+			radlog(L_ERR, "Dropping conflicting packet from "
+			       "client %s:%d - ID: %d due to unfinished request %d",
+			       client_name(request->packet->src_ipaddr),
+			       request->packet->src_port,
+			       request->packet->id,
+			       curreq->number);
+			request_free(&request);
+			return NULL;
+		}
+
+		/*
+		 *	We now check the authentication vectors.  If
+		 *	the client has sent us a request with
+		 *	identical code && ID, but different vector,
+		 *	then they MUST have gotten our response, so
+		 *	we can delete the original request, and
+		 *	process the new one.
+		 *
+		 *	If the vectors are the same, then it's a
+		 *	duplicate request, and we can send a
+		 *	duplicate reply.
 		 */
 		if (memcmp(curreq->packet->vector, request->packet->vector,
 				sizeof(request->packet->vector)) == 0) {
-			/*
-			 *  Maybe we've saved a reply packet.  If so,
-			 *  re-send it.  Otherwise, just complain.
-			 */
-			if ((curreq->reply) && (curreq->reply->code != 0)) {
-				radlog(L_INFO, "Sending duplicate authentication reply"
-						" to client %s:%d - ID: %d", client_name(curreq->packet->src_ipaddr),
-						curreq->packet->src_port, curreq->packet->id);
-
-				rad_send(curreq->reply, curreq->packet, curreq->secret);
-
-				/*
-				 *  There's no reply, but maybe there's
-				 *  an outstanding proxy request.
-				 *
-				 *  If so, then kick the proxy again.
-				 */
-			} else if (curreq->proxy != NULL) {
-				if (proxy_synchronous) {
-					if (!curreq->proxy_reply) {
-						DEBUG2("Sending duplicate proxy request to client %s:%d - ID: %d",
-						       client_name(curreq->proxy->dst_ipaddr), request->packet->src_port,
-						       curreq->proxy->id);
-						
-						curreq->proxy_next_try = request->timestamp + proxy_retry_delay;
-						rad_send(curreq->proxy, curreq->packet, curreq->proxysecret);
-					} else {
-						DEBUG2("Ignoring duplicate authentication packet"
-						       " from client %s:%d - ID: %d, as the proxy reply is currently being processed.",
-						       client_name(request->packet->src_ipaddr),
-						       request->packet->src_port,
-						       request->packet->id);
-					}
-				} else {
-					DEBUG2("Ignoring duplicate authentication packet"
-							" from client %s:%d - ID: %d, due to outstanding proxy request.",
-							client_name(request->packet->src_ipaddr),
-							request->packet->src_port,
-							request->packet->id);
-				}
-			} else {
-				/*
-				 *  This request wasn't proxied.
-				 */
-				radlog(L_ERR, "Dropping duplicate authentication packet"
-						" from client %s:%d - ID: %d", client_name(request->packet->src_ipaddr),
-						request->packet->src_port, request->packet->id);
-			}
+			rad_assert(curreq->reply != NULL);
 
 			/*
-			 *  Delete the duplicate request.
-			 */
-			request_free(&request);
-			return NULL;
-			
-			/*
-			 *  The packet vectors are different, so
-			 *  we can delete the old request from
-			 *  the list.
-			 */
-		} else if (curreq->finished) {
-			if (last_request == curreq) {
-				last_request = rl_next(last_request);
-			}
-
-			/*
-			 *	If we're keeping a delayed reject, and we
-			 *	get a new request, then we send the reject
-			 *	before deleting it.
-			 */
-			if ((curreq->options & RAD_REQUEST_OPTION_DELAYED_REJECT) != 0) {
-			  curreq->options &= ~RAD_REQUEST_OPTION_DELAYED_REJECT;
-			  rad_send(curreq->reply, curreq->packet,
-				   curreq->secret);
-			}
-
-			rl_delete(curreq);
-
-			/*
-			 *  ??? the client sent us a new request
-			 *  with the same ID, while we were
-			 *  processing the old one!  What should
-			 *  we do?
+			 *	If the packet has been delayed, then
+			 *	silently send a response, and clear the
+			 *	delayed flag.
 			 *
-			 *  Right now, we just drop the new packet..
+			 *	Note that this means if the NAS kicks
+			 *	us while we're delaying a reject, then
+			 *	the reject may be sent sooner than
+			 *	otherwise.
+			 *
+			 *	This COULD be construed as a bug.
+			 *	Maybe what we want to do is to ignore
+			 *	the duplicate packet, and send the
+			 *	reject later.
 			 */
-		} else {
-			radlog(L_ERR, "Dropping conflicting authentication packet"
-					" from client %s:%d - ID: %d",
-					client_name(request->packet->src_ipaddr),
-					request->packet->src_port,
-					request->packet->id);
+			if (curreq->options & RAD_REQUEST_OPTION_DELAYED_REJECT) {
+				curreq->options &= ~RAD_REQUEST_OPTION_DELAYED_REJECT;
+				rad_send(curreq->reply, curreq->packet, curreq->secret);
 				request_free(&request);
 				return NULL;
+			}
+
+			/*
+			 *	Maybe we've saved a reply packet.  If
+			 *	so, re-send it.  Otherwise, just
+			 *	complain.
+			 */
+			if (curreq->reply->code != 0) {
+				DEBUG2("Sending duplicate reply "
+				       "to client %s:%d - ID: %d",
+				       client_name(curreq->packet->src_ipaddr),
+				       curreq->packet->src_port, curreq->packet->id);
+				rad_send(curreq->reply, curreq->packet, curreq->secret);
+				request_free(&request);
+				return NULL;
+			}
+
+			/*
+			 *	At this point, there isn't a live
+			 *	thread handling the old request.  The
+			 *	old request isn't finished, AND
+			 *	there's no reply for it.
+			 *
+			 *	Therefore, we MUST be waiting for a reply
+			 *	from the proxy.
+			 *
+			 *	If not, then we have no clue what to
+			 *	do, so we drop the new request, and
+			 *	hope that the NAS doesn't bug us about
+			 *	it.
+			 */
+			if (!curreq->proxy) {
+				radlog(L_ERR, "Dropping packet from client "
+				       "%s:%d - ID: %d due to dead request %d",
+				       client_name(request->packet->src_ipaddr),
+				       request->packet->src_port,
+				       request->packet->id,
+				       curreq->number);
+				request_free(&request);
+				return NULL;
+			}
+
+			/*
+			 *	If there IS a reply from the proxy,
+			 *	then curreq SHOULD be marked alive, OR
+			 *	there should have been a reply sent to
+			 *	the NAS.  If none of these is true, then
+			 *	we don't know what to do, so we drop the
+			 *	request.
+			 */
+			if (curreq->proxy_reply) {
+				radlog(L_ERR, "Dropping packet from client "
+				       "%s:%d - ID: %d due to confused proxied request %d",
+				       client_name(request->packet->src_ipaddr),
+				       request->packet->src_port,
+				       request->packet->id,
+				       curreq->number);
+				request_free(&request);
+				return NULL;
+			}
+
+			/*
+			 *	We're taking care of sending duplicate
+			 *	proxied packets, so we ignore any duplicate
+			 *	requests from the NAS.
+			 */
+			if (!proxy_synchronous) {
+				DEBUG2("Ignoring duplicate packet from client "
+				       "%s:%d - ID: %d, due to outstanding proxied request %d.",
+				       client_name(request->packet->src_ipaddr),
+				       request->packet->src_port,
+				       request->packet->id,
+				       curreq->number);
+				
+				request_free(&request);
+				return NULL;
+			}
+
+			/*
+			 *	We ARE proxying the request, and we
+			 *	have NOT received a proxy reply yet,
+			 *	and we ARE doing synchronous proxying.
+			 *
+			 *	In that case, go kick the home RADIUS
+			 *	server again.
+			 */
+			DEBUG2("Sending duplicate proxied request to client %s:%d - ID: %d",
+			       client_name(curreq->proxy->dst_ipaddr), request->packet->src_port,
+			       curreq->proxy->id);
+			
+			curreq->proxy_next_try = request->timestamp + proxy_retry_delay;
+			rad_send(curreq->proxy, curreq->packet, curreq->proxysecret);
+			request_free(&request);
+			return NULL;
+		} /* else the vectors were different. */
+
+		/*
+		 *	If we're keeping a delayed reject, and we
+		 *	get a new request, then we discard the reject,
+		 *	in order to not confuse the NAS with an old
+		 *	response to a new request.
+		 */
+		
+		/*
+		 *	Fix up stuff.
+		 */
+		if (last_request == curreq) {
+			last_request = rl_next(last_request);
 		}
+
+		rl_delete(curreq);
 	} /* a similar packet already exists. */
 
 	/*
@@ -2071,8 +2176,8 @@ static int rad_spawn_child(REQUEST *request, RAD_REQUEST_FUNP fun)
 	 */
 	child_pid = fork();
 	if (child_pid < 0) {
-		radlog(L_ERR, "Fork failed for request from nas %s - ID: %d",
-				nas_name2(request->packet),
+		radlog(L_ERR, "Fork failed for request from client %s - ID: %d",
+				client_name(request->packet->src_ipaddr),
 				request->packet->id);
 		retval = -1;
 		goto exit_child_critsec;
@@ -2111,7 +2216,6 @@ static int sig_cleanup_walker(REQUEST *req, void *data)
 	req->finished = TRUE;
 	return 0;
 }
-#endif /* HAVE_PTHREAD_H */
 
 /* used in critical section */
 void queue_sig_cleanup(int sig) {
@@ -2119,6 +2223,7 @@ void queue_sig_cleanup(int sig) {
 	needs_child_cleanup++;
 	return;
 }
+#endif /* HAVE_PTHREAD_H */
 
 
 /*ARGSUSED*/
@@ -2167,7 +2272,9 @@ void sig_cleanup(int sig)
 		 *	are from Exec-Program.  We don't care about them,
 		 *	so once we've grabbed their PID's, we're done.
 		 */
-#ifndef HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
+		rad_savepid(pid, status);
+#else
 		/*
 		 *  Loop over ALL of the active requests, looking
 		 *  for the one which caused the signal.
@@ -2283,55 +2390,81 @@ static REQUEST *proxy_check_list(REQUEST *request)
 	REQUEST *oldreq;
 	
 	/*
-	 *  Find the original request in the request list
+	 *	Find the original request in the request list
 	 */
 	oldreq = rl_find_proxy(request);
-	if (oldreq) {
-		/*
-		 *  If there is already a reply,
-		 *  maybe the new one is a duplicate?
-		 */
-		if (oldreq->proxy_reply) {
-			if (memcmp(oldreq->proxy_reply->vector,
-					request->packet->vector,
-					sizeof(oldreq->proxy_reply->vector)) == 0) {
-				DEBUG2("Ignoring duplicate proxy reply");
-				request_free(&request);
-				return NULL;
-			} else {
+
+	/*
+	 *	If we haven't found the original request which was
+	 *	sent, to get this reply.  Complain, and discard this
+	 *	request, as there's no way for us to send it to a NAS.
+	 */
+	if (!oldreq) {
+		radlog(L_PROXY, "No matching request was found for proxy reply from server %s:%d - ID %d",
+		       client_name(request->packet->src_ipaddr),
+		       request->packet->src_port,
+		       request->packet->id);
+		request_free(&request);
+		return NULL;
+	}
+
+	/*
+	 *	If the request (duplicate or now) is currently
+	 *	being processed, then discard the new request.
+	 */
+	if (oldreq->child_pid != NO_SUCH_CHILD_PID) {
+		radlog(L_ERR, "Discarding duplicate reply from home server %s:%d - ID: %d due to live request %d",
+		       client_name(request->packet->src_ipaddr),
+		       request->packet->src_port,
+		       request->packet->id,
+		       oldreq->number);
+		request_free(&request);
+		return NULL;
+	}
+	
+	/*
+	 *	The proxy reply has arrived too late, as the original
+	 *	(old) request has timed out, been rejected, and marked
+	 *	as finished.  The client has already received a
+	 *	response, so there is nothing that can be done. Delete
+	 *	the tardy reply from the home server, and return NULL.
+	 */
+	if ((oldreq->reply->code != 0) ||
+	    (oldreq->finished)) {
+		radlog(L_ERR, "Reply from home server %s:%d arrived too late for request %d. Try increasing 'retry_delay' or 'max_request_time'",
+		       client_name(request->packet->src_ipaddr),
+		       request->packet->src_port,
+		       oldreq->number);
+		request_free(&request);
+		return NULL;
+	}
+	
+	/*
+	 *	If there is already a reply, maybe this one is a
+	 *	duplicate?
+	 */
+	if (oldreq->proxy_reply) {
+		if (memcmp(oldreq->proxy_reply->vector,
+			   request->packet->vector,
+			   sizeof(oldreq->proxy_reply->vector)) == 0) {
+			DEBUG2("Ignoring duplicate proxy reply");
+		} else {
 				/*
 				 *  ??? The home server gave us a new
 				 *  proxy reply, which doesn't match
 				 *  the old one.  Delete it!
 				 */
-				DEBUG2("Ignoring conflicting proxy reply");
-				request_free(&request);
-				return NULL;
-			}
-		} else if ((oldreq->reply->code != 0) ||
-			   (oldreq->finished)) {
-			/*
-			 *	The proxy reply has arrived too late,
-			 *	the old request has timed out and is
-			 *	finished.  The client has received a
-			 *	response, so there is nothing that can
-			 *	be done. Delete the reply, and return
-			 *	NULL.
-			 */
-			radlog(L_ERR, "Proxy reply arrived too late. Try increasing retry_delay");
-			request_free(&request);
-			return NULL;
- 		}
-	} else {
+			DEBUG2("Ignoring conflicting proxy reply");
+		}
+		
 		/*
-		 *  If we haven't found the old request, complain.
+		 *	We've already received a reply, so
+		 *	we discard this one, as we don't want
+		 *	to do duplicate work.
 		 */
-		radlog(L_PROXY, "No request found for proxy reply from server %s - ID %d",
-				client_name(request->packet->src_ipaddr),
-				request->packet->id);
 		request_free(&request);
 		return NULL;
-	}
+	} /* else there wasn't a proxy reply yet, so we can process it */
 
 	/*
 	 *  Refresh the old request, and update it with the proxy reply.
@@ -2383,11 +2516,9 @@ static int refresh_request(REQUEST *request, void *data)
 	}
 
 	/*
-	 *  If the request has finished processing,
-	 *  AND it's child has been cleaned up,
-	 *  AND it's time to clean up the request,
-	 *      OR, it's an accounting request.
-	 *  THEN, go delete it.
+	 *  If the request has finished processing, AND it's child has
+	 *  been cleaned up, AND it's time to clean up the request,
+	 *  OR, it's an accounting request.  THEN, go delete it.
 	 *
 	 *  If this is an accounting request, we delete it
 	 *  immediately, as there CANNOT be duplicate accounting
@@ -2414,8 +2545,7 @@ static int refresh_request(REQUEST *request, void *data)
 	}
 
 	/*
-	 *  Maybe the child process
-	 *  handling the request has hung:
+	 *  Maybe the child process handling the request has hung:
 	 *  kill it, and continue.
 	 */
 	if ((request->timestamp + max_request_time) <= info->now) {
@@ -2424,6 +2554,34 @@ static int refresh_request(REQUEST *request, void *data)
 		child_pid = request->child_pid;
 		number = request->number;
 
+		/*
+		 *	There MUST be a RAD_PACKET reply.
+		 */
+		rad_assert(request->reply != NULL);
+
+		/*
+		 *	If we've tried to proxy the request, and
+		 *	the proxy server hasn't responded, then
+		 *	we send a REJECT back to the caller.
+		 *
+		 *	For safety, we assert that there is no child
+		 *	handling the request.  If the assertion fails,
+		 *	it means that we've sent a proxied request to
+		 *	the home server, and the child thread is still
+		 *	sitting on the request!
+		 */
+		if (request->proxy && !request->proxy_reply) {
+			rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
+			
+			radlog(L_ERR, "Rejecting request %d due to lack of any response from home server %s:%d",
+			       request->number,
+			       client_name(request->packet->src_ipaddr),
+			       request->packet->src_port);
+			rad_reject(request);
+			request->finished = TRUE;
+			return RL_WALK_CONTINUE;
+		}
+
 		if (kill_unresponsive_children) {
 			if (child_pid != NO_SUCH_CHILD_PID) {
 				/*
@@ -2524,7 +2682,7 @@ static int refresh_request(REQUEST *request, void *data)
 	 */
 	if (request->proxy_try_count == 0) {
 		rad_reject(request);
-		realm_disable(request->proxy->dst_ipaddr);
+		realm_disable(request->proxy->dst_ipaddr,request->proxy->dst_port);
 		request->finished = TRUE;
 		goto setup_timeout;
 	}
