@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.38 2002/02/08 16:52:12 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.39 2002/04/23 15:06:30 cparker Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.38 2002/02/08 16:52:12 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.39 2002/04/23 15:06:30 cparker Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -77,6 +77,14 @@ static int paircompare(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
 		return -2;
 #endif
 
+	/*
+	 *      Check for =* and !* and return appropriately
+	 */
+	if( check->operator == T_OP_CMP_TRUE )
+	         return 0;  /* always return 0/EQUAL */
+	if( check->operator == T_OP_CMP_FALSE )
+	         return 1;  /* always return 1/NOT EQUAL */
+
 	/*
 	 *	See if there is a special compare function.
 	 */
@@ -275,6 +283,8 @@ int paircmp(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check, VALUE_PAIR **r
 				radlog(L_ERR,  "Invalid operator for item %s: "
 						"reverting to '=='", check_item->name);
 				/*FALLTHRU*/
+		        case T_OP_CMP_TRUE:    /* compare always == 0 */
+		        case T_OP_CMP_FALSE:   /* compare always == 1 */
 			case T_OP_CMP_EQ:
 				if (compare != 0) result = -1;
 				break;
