@@ -1,7 +1,7 @@
 /*
  * rlm_detail.c	accounting:    Write the "detail" files.
  *
- * Version:	$Id: rlm_detail.c,v 1.25 2001/11/07 15:55:14 aland Exp $
+ * Version:	$Id: rlm_detail.c,v 1.26 2002/02/01 19:39:58 aland Exp $
  *
  *  This program is is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2 if the
@@ -19,7 +19,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: rlm_detail.c,v 1.25 2001/11/07 15:55:14 aland Exp $";
+static const char rcsid[] = "$Id: rlm_detail.c,v 1.26 2002/02/01 19:39:58 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -167,7 +167,7 @@ static int detail_accounting(void *instance, REQUEST *request)
 		close(outfd);
 	} else {
 		/* Post a timestamp */
-		fputs(ctime(&request->timestamp), outfp);
+		fputs(ctime_r(&request->timestamp, buffer), outfp);
 
 		/* Write each attribute/value to the log file */
 		pair = request->packet->vps;
