@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id: eap_tls.c,v 1.5 2002/02/26 00:56:35 raghu Exp $
+ * Version:     $Id: eap_tls.c,v 1.8 2002/04/26 21:52:03 raghu Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -68,6 +68,8 @@ void eaptls_free(EAPTLS_PACKET **eaptls_packet_ptr)
 		free(eaptls_packet->data);
 		eaptls_packet->data = NULL;
 	}
+
+	free(eaptls_packet);
 	*eaptls_packet_ptr = NULL;
 }
 
@@ -201,6 +203,7 @@ int eaptls_request(EAP_DS *eap_ds, tls_session_t *ssn)
 
 	eaptls_compose(eap_ds, &reply);
 	free(reply.data);
+	reply.data = NULL;
 
 	return 1;
 }
@@ -219,6 +222,7 @@ eaptls_status_t eaptls_ack_handler(EAP_HANDLER *handler)
 
 	tls_session = (tls_session_t *)handler->opaque;
 	if ((tls_session == NULL) || (tls_session->info.origin == 0)) {
+
 		radlog(L_ERR, "rlm_eap_tls: Unexpected ACK received");
 		return EAPTLS_NOOP;
 	}
@@ -235,11 +239,15 @@ eaptls_status_t eaptls_ack_handler(EAP_HANDLER *handler)
 			eaptls_success(handler->eap_ds);
 			session_free(&handler->opaque);
 			return EAPTLS_SUCCESS;
-		} else {
+		} else if (tls_session->fragment > 0) {
 			/* Fragmentation handler, send next fragment */
 			eaptls_request(handler->eap_ds, tls_session);
 			return EAPTLS_REQUEST;
 		}
+		/*
+		 * For the rest of the conditions,
+		 * switch over to the default section below.
+		 */
 
 	default:
 		radlog(L_ERR, "rlm_eap_tls: Invalid ACK received");
@@ -313,10 +321,10 @@ eaptls_status_t eaptls_verify(EAP_DS *eap_ds, EAP_DS *prev_eap_ds)
 		((eaptls_packet != NULL) && (eaptls_packet->flags == 0x00))) {
 
 		if (prev_eap_ds->request->id == eap_ds->response->id) {
-			radlog(L_INFO, "rlm_eap_tls:  Received EAP-TLS ACK message");
+			radlog(L_INFO, "rlm_eap_tls: Received EAP-TLS ACK message");
 			return EAPTLS_ACK;
 		} else {
-			radlog(L_ERR, "rlm_eap_tls:  Received Invalid EAP-TLS ACK message");
+			radlog(L_ERR, "rlm_eap_tls: Received Invalid EAP-TLS ACK message");
 			return EAPTLS_INVALID;
 		}
 	}
@@ -583,9 +591,12 @@ void eaptls_operation(EAPTLS_PACKET *eaptls_packet, eaptls_status_t status, EAP_
 		 * Authenticate the user and send Success/Failure
 		 * If more info is required then send another request.
 		 */
-		tls_handshake_recv(tls_session);
-		eaptls_request(handler->eap_ds, tls_session);
-		record_init(&tls_session->dirty_in);
+		if (tls_handshake_recv(tls_session)) {
+			eaptls_request(handler->eap_ds, tls_session);
+		} else {
+			eaptls_fail(handler->eap_ds);
+		}
+		//record_init(&tls_session->dirty_in);
 	}
 	return;
 }
