@@ -50,8 +50,31 @@
  *	  a call to ldap_release_conn when it has finished.
  *	- Request only the user attributes that interest us (radius attributes,regular
  *	  profile,user password and access attribute).
+ * Mar 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Fixed a bug where the ldap server will kill the idle connections from the ldap
+ *	  connection pool. We now check if ldap_search returns LDAP_SERVER_DOWN and try to
+ *	  reconnect if it does. Bug noted by Dan Perik <dan_perik-work@ntm.org.pg>
+ * May 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Instead of the Group attribute we now have the Ldap-Group attribute, to avoid
+ *	  collisions with other modules
+ *	- If perform_search fails check the ld != NULL before using it. Based on a bug report
+ *	  by John <jhogenmiller@pennswoods.net>
+ * June 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Add the ability to do a paircmp on the check items. Add a compare_check_items boolean
+ *	  configuration directive which defaults to no. If it is set then we will do a compare
+ *	- Add another configuration directive. access_attr_used_for_allow. If it is set to yes
+ *	  then the access_attr will be used to allow user access. If it is set to no then it will
+ *	  be used to deny user access.
+ *	- Remember to free inst->atts in ldap_detach()
+ *	- Add a forgotten ldap_free_urldesc in ldap_xlat()
+ *	- Add a variable locked in the LDAP_CONN structure. We use this to avoid deadlocks. The mutex
+ *	  we are using is of type fast and can deadlock if the same thread tries to relock it. That
+ *	  could happen in case of calls to xlat.
+ *	- When ldap_search returns NO_SUCH_OBJECT don't return fail but notfound
+ * July 2002, Kostas Kalevras <kkalev@noc.ntua.gr>
+ *	- Fix the logic when we get an LDAP_SERVER_DOWN or we have conn->ld == NULL in perform_search
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.69 2002/02/21 00:03:03 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.80 2002/07/01 08:50:22 kkalev Exp $";
 
 #include "autoconf.h"
 
@@ -96,6 +119,7 @@ typedef struct TLDAP_RADIUS TLDAP_RADIUS;
 typedef struct ldap_conn {
 	LDAP		*ld;
 	char		bound;
+	char		locked;
 	pthread_mutex_t	mutex;
 } LDAP_CONN;
 
@@ -113,6 +137,8 @@ typedef struct {
 	int		num_conns;
 	int		cache_timeout;
 	int		cache_size;
+	int		do_comp;
+	int		default_allow;
 	char           *login;
 	char           *password;
 	char           *filter;
@@ -163,6 +189,8 @@ static CONF_PARSER module_config[] = {
 	{"dictionary_mapping", PW_TYPE_STRING_PTR, offsetof(ldap_instance,dictionary_mapping), NULL, "${confdir}/ldap.attrmap"},
 	{"ldap_debug", PW_TYPE_INTEGER, offsetof(ldap_instance,ldap_debug), NULL, "0x0000"},
 	{"ldap_connections_number", PW_TYPE_INTEGER, offsetof(ldap_instance,num_conns), NULL, "5"},
+	{"compare_check_items", PW_TYPE_BOOLEAN, offsetof(ldap_instance,do_comp), NULL, "no"},
+	{"access_attr_used_for_allow", PW_TYPE_BOOLEAN, offsetof(ldap_instance,default_allow), NULL, "yes"},
 
 	{NULL, -1, 0, NULL, NULL}
 };
@@ -186,8 +214,9 @@ static inline int ldap_get_conn(LDAP_CONN *conns,LDAP_CONN **ret,void *instance)
 	register int i = 0;
 
 	for(;i<inst->num_conns;i++){
-		if (pthread_mutex_trylock(&(conns[i].mutex)) == 0){
+		if (conns[i].locked == 0 && pthread_mutex_trylock(&(conns[i].mutex)) == 0){
 			*ret = &conns[i];
+			conns[i].locked = 1;
 			DEBUG("ldap_get_conn: Got Id: %d",i);
 			return i;
 		}
@@ -199,6 +228,7 @@ static inline int ldap_get_conn(LDAP_CONN *conns,LDAP_CONN **ret,void *instance)
 static inline void ldap_release_conn(int i, LDAP_CONN *conns)
 {
 	DEBUG("ldap_release_conn: Release Id: %d",i);
+	conns[i].locked = 0;
 	pthread_mutex_unlock(&(conns[i].mutex));
 }
 
@@ -242,10 +272,7 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	inst->check_item_map = NULL;
 	inst->conns = NULL;
 
-	paircompare_register(PW_GROUP, PW_USER_NAME, ldap_groupcmp, inst);
-#ifdef PW_GROUP_NAME /* compat */
-	paircompare_register(PW_GROUP_NAME, PW_USER_NAME, ldap_groupcmp, inst);
-#endif
+	paircompare_register(PW_LDAP_GROUP, PW_USER_NAME, ldap_groupcmp, inst);
 	DEBUG("conns: %p",inst->conns);
 
 	xlat_name = cf_section_name2(conf);
@@ -269,6 +296,7 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	}
 	for(;i<inst->num_conns;i++){
 		inst->conns[i].bound = 0;
+		inst->conns[i].locked = 0;
 		inst->conns[i].ld = NULL;
 		pthread_mutex_init(&inst->conns[i].mutex, NULL);
 	}	
@@ -447,6 +475,7 @@ perform_search(void *instance, LDAP_CONN *conn, char *search_basedn, int scope,
 	int             res = RLM_MODULE_OK;
 	int		ldap_errno = 0;
 	ldap_instance  *inst = instance;
+	int		search_retry = 0;
 
 	*result = NULL;
 
@@ -454,7 +483,8 @@ perform_search(void *instance, LDAP_CONN *conn, char *search_basedn, int scope,
 		radlog(L_ERR, "rlm_ldap: NULL connection handle passed");
 		return RLM_MODULE_FAIL;
 	}
-	if (!conn->bound) {
+retry:
+	if (!conn->bound || conn->ld == NULL) {
 		DEBUG2("rlm_ldap: attempting LDAP reconnection");
 		if (conn->ld){
 			DEBUG2("rlm_ldap: closing existing LDAP connection");
@@ -471,8 +501,19 @@ perform_search(void *instance, LDAP_CONN *conn, char *search_basedn, int scope,
 	DEBUG2("rlm_ldap: performing search in %s, with filter %s", search_basedn ? search_basedn : "(null)" , filter);
 	switch (ldap_search_st(conn->ld, search_basedn, scope, filter, attrs, 0, &(inst->timeout), result)) {
 	case LDAP_SUCCESS:
+	case LDAP_NO_SUCH_OBJECT:
 		break;
-
+	case LDAP_SERVER_DOWN:
+		radlog(L_ERR, "rlm_ldap: ldap_search() failed: LDAP connection lost.");
+		if (search_retry == 0){
+			radlog(L_INFO, "rlm_ldap: Attempting reconnect");
+			search_retry = 1;
+			conn->bound = 0;
+			ldap_msgfree(*result);	
+			goto retry;
+		}
+		ldap_msgfree(*result);
+		return RLM_MODULE_FAIL;
 	default:
 		ldap_get_option(conn->ld, LDAP_OPT_ERROR_NUMBER, &ldap_errno);
 		radlog(L_ERR, "rlm_ldap: ldap_search() failed: %s", ldap_err2string(ldap_errno));
@@ -491,7 +532,7 @@ perform_search(void *instance, LDAP_CONN *conn, char *search_basedn, int scope,
 
 
 /*
- * ldap_groupcmp(). Implement the Group == "group" filter
+ * ldap_groupcmp(). Implement the Ldap-Group == "group" filter
  */
 
 static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
@@ -572,11 +613,11 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
 		return 1;
 	}
 
-	if (inst->cache_timeout >0)
+	if (inst->cache_timeout >0 && conn->ld != NULL)
 		ldap_enable_cache(conn->ld, inst->cache_timeout, inst->cache_size);
 
         if ((res = perform_search(inst, conn, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK){
-		if (inst->cache_timeout >0)
+		if (inst->cache_timeout >0 && conn->ld != NULL)
 			ldap_disable_cache(conn->ld);
                 if (res == RLM_MODULE_NOTFOUND){
                         DEBUG("rlm_ldap::ldap_groupcmp: Group %s not found", (char *)check->strvalue);
@@ -616,7 +657,7 @@ static int ldap_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALU
         }
 
         if ((res = perform_search(inst, conn, group_dn, LDAP_SCOPE_BASE, filter, attrs, &result)) != RLM_MODULE_OK){
-		if (inst->cache_timeout >0)
+		if (inst->cache_timeout >0 && conn->ld != NULL)
 			ldap_disable_cache(conn->ld);
 		ldap_release_conn(conn_id,inst->conns);
                 if (res == RLM_MODULE_NOTFOUND){
@@ -690,7 +731,8 @@ static int ldap_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 	}
 	if ((conn_id = ldap_get_conn(inst->conns,&conn,inst)) == -1){
 		radlog(L_ERR, "rlm_ldap: All ldap connections are in use");
-		return 1;
+		ldap_free_urldesc(ldap_url);
+		return 0;
 	}
 	if ((res = perform_search(inst, conn, ldap_url->lud_dn, ldap_url->lud_scope, ldap_url->lud_filter, ldap_url->lud_attrs, &result)) != RLM_MODULE_OK){
 		if (res == RLM_MODULE_NOTFOUND){
@@ -764,9 +806,9 @@ ldap_authorize(void *instance, REQUEST * request)
 	int		res;
 	VALUE_PAIR	**check_pairs, **reply_pairs;
 	char		**vals;
-	VALUE_PAIR      *module_msg_vp;
+	VALUE_PAIR      *module_fmsg_vp;
 	VALUE_PAIR	*user_profile;
-	char            module_msg[MAX_STRING_LEN];
+	char            module_fmsg[MAX_STRING_LEN];
 	LDAP_CONN	*conn;
 	int		conn_id = -1;
 
@@ -809,9 +851,9 @@ ldap_authorize(void *instance, REQUEST * request)
 	if ((res = perform_search(instance, conn, basedn, LDAP_SCOPE_SUBTREE, filter, inst->atts, &result)) != RLM_MODULE_OK) {
 		DEBUG("rlm_ldap: search failed");
 		if (res == RLM_MODULE_NOTFOUND){
-			snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: User not found");
-			module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
-			pairadd(&request->packet->vps, module_msg_vp);
+			snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: User not found");
+			module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+			pairadd(&request->packet->vps, module_fmsg_vp);
 		}
 		ldap_release_conn(conn_id,inst->conns);
 		return (res);
@@ -839,26 +881,40 @@ ldap_authorize(void *instance, REQUEST * request)
 	/* Remote access is controled by attribute of the user object */
 	if (inst->access_attr) {
 		if ((vals = ldap_get_values(conn->ld, msg, inst->access_attr)) != NULL) {
-			DEBUG("rlm_ldap: checking if remote access for %s is allowed by %s", request->username->strvalue, inst->access_attr);
-			if (!strncmp(vals[0], "FALSE", 5)) {
-				DEBUG("rlm_ldap: dialup access disabled");
-				snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: Access Attribute denies access");
-				module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
-				pairadd(&request->packet->vps, module_msg_vp);
+			if (inst->default_allow){
+				DEBUG("rlm_ldap: checking if remote access for %s is allowed by %s", request->username->strvalue, inst->access_attr);
+				if (!strncmp(vals[0], "FALSE", 5)) {
+					DEBUG("rlm_ldap: dialup access disabled");
+					snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Access Attribute denies access");
+					module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+					pairadd(&request->packet->vps, module_fmsg_vp);
+					ldap_msgfree(result);
+					ldap_value_free(vals);
+					ldap_release_conn(conn_id,inst->conns);
+					return RLM_MODULE_USERLOCK;
+				}
+				ldap_value_free(vals);
+			}
+			else{
+				DEBUG("rlm_ldap: %s attribute exists - access denied by default", inst->access_attr);
+				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Access Attribute denies access");
+				module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_fmsg_vp);
 				ldap_msgfree(result);
 				ldap_value_free(vals);
 				ldap_release_conn(conn_id,inst->conns);
 				return RLM_MODULE_USERLOCK;
 			}
-			ldap_value_free(vals);
 		} else {
-			DEBUG("rlm_ldap: no %s attribute - access denied by default", inst->access_attr);
-			snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: Access Attribute denies access");
-			module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
-			pairadd(&request->packet->vps, module_msg_vp);
-			ldap_msgfree(result);
-			ldap_release_conn(conn_id,inst->conns);
-			return RLM_MODULE_USERLOCK;
+			if (inst->default_allow){
+				DEBUG("rlm_ldap: no %s attribute - access denied by default", inst->access_attr);
+				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Access Attribute denies access");
+				module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_fmsg_vp);
+				ldap_msgfree(result);
+				ldap_release_conn(conn_id,inst->conns);
+				return RLM_MODULE_USERLOCK;
+			}
 		}
 	}
 	if (inst->cache_timeout >0)
@@ -884,14 +940,14 @@ ldap_authorize(void *instance, REQUEST * request)
 			radlog (L_ERR, "rlm_ldap: unable to create filter.\n"); 
 
 		if ((res = perform_search(instance, conn, group, LDAP_SCOPE_BASE, filter, attrs, &gr_result)) != RLM_MODULE_OK) {
-			if (inst->cache_timeout >0)
+			if (inst->cache_timeout >0 && conn->ld != NULL)
 				ldap_disable_cache(conn->ld);
 			ldap_msgfree(result);
 			ldap_release_conn(conn_id,inst->conns);
 			if (res == RLM_MODULE_NOTFOUND){
-				snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: User is not an access group member");
-				module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
-				pairadd(&request->packet->vps, module_msg_vp);
+				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: User is not an access group member");
+				module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_fmsg_vp);
 				return (RLM_MODULE_USERLOCK);
 			}
 			else
@@ -1006,6 +1062,21 @@ ldap_authorize(void *instance, REQUEST * request)
 		pairadd(check_pairs, check_tmp);
 
 
+	DEBUG("rlm_ldap: looking for reply items in directory...");
+
+
+	if ((reply_tmp = ldap_pairget(conn->ld, msg, inst->reply_item_map,reply_pairs)) != NULL)
+		pairadd(reply_pairs, reply_tmp);
+
+       if (inst->do_comp && paircmp(request,request->packet->vps,*check_pairs,reply_pairs) != 0){
+		DEBUG("rlm_ldap: Pairs do not match. Rejecting user.");
+		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Pairs do not match");
+		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+		pairadd(&request->packet->vps, module_fmsg_vp);
+
+		return RLM_MODULE_REJECT;
+	}
+
 	/*
  	 * Module should default to LDAP authentication if no Auth-Type
 	 * specified
@@ -1014,13 +1085,6 @@ ldap_authorize(void *instance, REQUEST * request)
 		pairadd(check_pairs, pairmake("Auth-Type", "LDAP", T_OP_EQ));
 
 
-	DEBUG("rlm_ldap: looking for reply items in directory...");
-
-
-	if ((reply_tmp = ldap_pairget(conn->ld, msg, inst->reply_item_map,reply_pairs)) != NULL)
-		pairadd(reply_pairs, reply_tmp);
-
-
 	DEBUG("rlm_ldap: user %s authorized to use remote access",
 	      request->username->strvalue);
 	ldap_msgfree(result);
@@ -1047,8 +1111,8 @@ ldap_authenticate(void *instance, REQUEST * request)
 	char		basedn[1024];
 	int             res;
 	VALUE_PAIR     *vp_user_dn;
-	VALUE_PAIR      *module_msg_vp;
-	char            module_msg[MAX_STRING_LEN];
+	VALUE_PAIR      *module_fmsg_vp;
+	char            module_fmsg[MAX_STRING_LEN];
 	LDAP_CONN	*conn;
 	int		conn_id = -1;
 
@@ -1100,9 +1164,9 @@ ldap_authenticate(void *instance, REQUEST * request)
 		}
 		if ((res = perform_search(instance, conn, basedn, LDAP_SCOPE_SUBTREE, filter, attrs, &result)) != RLM_MODULE_OK) {
 			if (res == RLM_MODULE_NOTFOUND){
-				snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: User not found");
-				module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
-				pairadd(&request->packet->vps, module_msg_vp);
+				snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: User not found");
+				module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+				pairadd(&request->packet->vps, module_fmsg_vp);
 			}
 			ldap_release_conn(conn_id,inst->conns);
 			return (res);
@@ -1131,9 +1195,9 @@ ldap_authenticate(void *instance, REQUEST * request)
 	ld_user = ldap_connect(instance, user_dn, request->password->strvalue,
 			       1, &res);
 	if (ld_user == NULL){
-		snprintf(module_msg,MAX_STRING_LEN-1,"rlm_ldap: Bind as user failed");
-		module_msg_vp = pairmake("Module-Message", module_msg, T_OP_EQ);
-		pairadd(&request->packet->vps, module_msg_vp);
+		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_ldap: Bind as user failed");
+		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
+		pairadd(&request->packet->vps, module_fmsg_vp);
 		return (res);
 	}
 
@@ -1308,10 +1372,15 @@ ldap_detach(void *instance)
 		pair = nextpair;
 	}
 
-	paircompare_unregister(PW_GROUP, ldap_groupcmp);
-#ifdef PW_GROUP_NAME
-	paircompare_unregister(PW_GROUP_NAME, ldap_groupcmp);
-#endif
+	if (inst->atts){
+		int i = 0;
+
+		while(inst->atts[i])
+			free(inst->atts[i++]);
+		free(inst->atts);
+	}
+
+	paircompare_unregister(PW_LDAP_GROUP, ldap_groupcmp);
 	xlat_unregister(inst->xlat_name,ldap_xlat);
 	free(inst->xlat_name);
 
@@ -1393,7 +1462,7 @@ ldap_pairget(LDAP * ld, LDAPMessage * entry,
 
 				if (is_generic_attribute) {
 					/* this is a generic attribute */
-					int dummy; /* makes pairread happy */
+					LRAD_TOKEN dummy; /* makes pairread happy */
 					
 					/* not sure if using pairread here is ok ... */
 					if ( (newpair = pairread(&ptr, &dummy)) != NULL) {
