@@ -7,7 +7,7 @@
  *		That, in fact, was again based on the original stuff
  *		from Jeph Blaize <jblaize@kiva.net> done in May 1997.
  *
- * Version:	$Id: rlm_pam.c,v 1.24 2001/07/25 18:07:01 aland Exp $
+ * Version:	$Id: rlm_pam.c,v 1.25 2002/01/13 11:12:05 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -233,10 +233,10 @@ static int pam_auth(void *instance, REQUEST *request)
 
 	/*
 	 *	We can only authenticate user requests which HAVE
-	 *	a Password attribute.
+	 *	a User-Password attribute.
 	 */
 	if (!request->password) {
-		radlog(L_AUTH, "rlm_pam: Attribute \"Password\" is required for authentication.");
+		radlog(L_AUTH, "rlm_pam: Attribute \"User-Password\" is required for authentication.");
 		return RLM_MODULE_INVALID;
 	}
 
@@ -245,7 +245,7 @@ static int pam_auth(void *instance, REQUEST *request)
 	 *  and not anything else.
 	 */
 	if (request->password->attribute != PW_PASSWORD) {
-		radlog(L_AUTH, "rlm_pam: Attribute \"Password\" is required for authentication.  Cannot use \"%s\".", request->password->name);
+		radlog(L_AUTH, "rlm_pam: Attribute \"User-Password\" is required for authentication.  Cannot use \"%s\".", request->password->name);
 		return RLM_MODULE_INVALID;
 	}
 
