@@ -3,7 +3,7 @@
  *		Contains the functions for the "huntgroups" and "hints"
  *		files.
  *
- * Version:     $Id: rlm_preprocess.c,v 1.35 2001/11/09 00:53:10 kkalev Exp $
+ * Version:     $Id: rlm_preprocess.c,v 1.37 2002/02/21 19:11:01 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.35 2001/11/09 00:53:10 kkalev Exp $";
+static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.37 2002/02/21 19:11:01 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -497,6 +497,27 @@ static int huntgroup_access(PAIR_LIST *huntgroups, VALUE_PAIR *request_pairs)
 		 */
 		r = RLM_MODULE_REJECT;
 		if (hunt_paircmp(request_pairs, i->reply) == 0) {
+			VALUE_PAIR *vp;
+
+			/*
+			 *  We've matched the huntgroup, so add it in
+			 *  to the list of request pairs.
+			 */
+			vp = pairfind(request_pairs, PW_HUNTGROUP_NAME);
+			if (!vp) {
+				vp = paircreate(PW_HUNTGROUP_NAME,
+						PW_TYPE_STRING);
+				if (!vp) {
+					radlog(L_ERR, "No memory");
+					exit(1);
+				}
+				
+				strNcpy(vp->strvalue, i->name,
+					sizeof(vp->strvalue));
+				vp->length = strlen(vp->strvalue);
+
+				pairadd(&request_pairs, vp);
+			}
 			r = RLM_MODULE_OK;
 		}
 		break;
@@ -644,7 +665,25 @@ static int preprocess_authorize(void *instance, REQUEST *request)
 	add_nas_attr(request);
 
 	hints_setup(data->hints, request);
-	
+
+	/*
+	 *      If there is a PW_CHAP_PASSWORD attribute but there
+	 *      is PW_CHAP_CHALLENGE we need to add it so that other
+	 *	modules can use it as a normal attribute.
+	 */
+	if (pairfind(request->packet->vps, PW_CHAP_PASSWORD) &&
+	    pairfind(request->packet->vps, PW_CHAP_CHALLENGE) == NULL) {
+		VALUE_PAIR *vp;
+		vp = paircreate(PW_CHAP_CHALLENGE, PW_TYPE_OCTETS);
+		if (!vp) {
+			radlog(L_ERR|L_CONS, "no memory");
+			exit(1);
+		}
+		vp->length = AUTH_VECTOR_LEN;
+		memcpy(vp->strvalue, request->packet->vector, AUTH_VECTOR_LEN);
+		pairadd(&request->packet->vps, vp);
+	}
+
 	if (huntgroup_access(data->huntgroups, request->packet->vps) != RLM_MODULE_OK) {
 		radlog(L_AUTH, "No huntgroup access: [%s] (%s)",
 		    request->username->strvalue,
