@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.75 2002/03/02 16:13:49 kkalev Exp $
+ * Version:	$Id: rlm_sql.c,v 1.87 2002/06/10 10:32:17 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.75 2002/03/02 16:13:49 kkalev Exp $";
+	"$Id: rlm_sql.c,v 1.87 2002/06/10 10:32:17 fcusack Exp $";
 
 #include "autoconf.h"
 
@@ -81,6 +81,9 @@ static CONF_PARSER module_config[] = {
 	{"accounting_stop_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_stop_query), NULL, ""},
 	{"accounting_stop_query_alt", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_stop_query_alt), NULL, ""},
 	{"connect_failure_retry_delay", PW_TYPE_INTEGER, offsetof(SQL_CONFIG,connect_failure_retry_delay), NULL, "60"},
+	{"simul_count_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_count_query), NULL, ""},
+	{"simul_verify_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_verify_query), NULL, ""},
+	{"simul_zap_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_zap_query), NULL, ""},
 
 	{NULL, -1, 0, NULL, NULL}
 };
@@ -101,10 +104,9 @@ static int rlm_sql_init(void) {
 }
 
 /*
- * sql xlat function. Right now only SELECTs are supported. Only the first element of the
- * SELECT result will be used.
+ *	sql xlat function. Right now only SELECTs are supported. Only
+ *	the first element of the SELECT result will be used.
  */
-
 static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int freespace,
 			RADIUS_ESCAPE_STRING func)
 {
@@ -112,7 +114,7 @@ static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 	SQL_ROW row;
 	SQL_INST *inst=instance;
 	char querystr[MAX_QUERY_LEN];
-	int ret=0;
+	int ret = 0;
 
 	DEBUG("rlm_sql: - sql_xlat");
 	/*
@@ -153,10 +155,88 @@ static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 	return ret;
 }
 
+/*
+ *	Translate the SQL queries.
+ */
+static int sql_escape_func(char *out, int outlen, const char *in)
+{
+	int len = 0;
+	
+	while (in[0]) {
+		/*
+		 *  Only one byte left.
+		 */
+		if (outlen <= 1) {
+			break;
+		}
+		
+		/*
+		 *	Non-printable characters get replaced with their
+		 *	mime-encoded equivalents.
+		 */
+		if ((in[0] < 32) ||
+		    strchr("@abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.-_: ", *in) == NULL) {
+			snprintf(out, outlen, "=%02X", (unsigned char) in[0]);
+			in++;
+			out += 3;
+			outlen -= 3;
+			len += 3;
+			continue;
+		}
+		
+		/*
+		 *	Else it's a nice character.
+		 */
+		*out = *in;
+		out++;
+		in++;
+		outlen--;
+		len++;
+	}
+	*out = '\0';
+	return len;
+}
+
+/*
+ *	Set the SQl user name.
+ */
+static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username) {
+	VALUE_PAIR *vp=NULL;
+	char tmpuser[MAX_STRING_LEN];
+
+	tmpuser[0]=0;
+	sqlusername[0]=0;
+
+	/* Remove any user attr we added previously */
+	pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
+
+	if (username != NULL) {
+		strNcpy(tmpuser, username, MAX_STRING_LEN);
+	} else if (strlen(inst->config->query_user)) {
+		radius_xlat(tmpuser, MAX_STRING_LEN, inst->config->query_user, request, sql_escape_func);
+	} else {
+		return 0;
+	}
+
+	if (*tmpuser) {
+		strNcpy(sqlusername, tmpuser, MAX_STRING_LEN * 2);
+		DEBUG2("sql_set_user:  escaped user --> '%s'", sqlusername);
+		vp = pairmake("SQL-User-Name", sqlusername, 0);
+		if (vp == NULL) {
+			radlog(L_ERR, "%s", librad_errstr);
+			return -1;
+		}
+
+		pairadd(&request->packet->vps, vp);
+		return 0;
+	}
+	return -1;
+}
+
 static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 
 	SQL_INST *inst;
-	lt_dlhandle *handle;
+	lt_dlhandle handle;
 	char *xlat_name;
 
 	inst = rad_malloc(sizeof(SQL_INST));
@@ -270,15 +350,15 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	/*
 	 * Set, escape, and check the user attr here
 	 */
-	if (sql_set_user(inst, request, sqlusername, 0) < 0)
+	if (sql_set_user(inst, request, sqlusername, NULL) < 0)
 		return RLM_MODULE_FAIL;
-	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_check_query, request, NULL);
+	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_check_query, request, sql_escape_func);
 
 	sqlsocket = sql_get_socket(inst);
 	if (sqlsocket == NULL) {
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
-		return(RLM_MODULE_NOOP);
+		return(RLM_MODULE_FAIL);
 	}
 
 	found = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_USERDATA);
@@ -286,41 +366,53 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	 *      Find the entry for the user.
 	 */
 	if (found > 0) {
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, NULL);
+		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, sql_escape_func);
 		sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_reply_query, request, NULL);
+		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_reply_query, request, sql_escape_func);
 		sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_USERDATA);
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, NULL);
+		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, sql_escape_func);
 		sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
 	} else if (found < 0) {
 		radlog(L_ERR, "rlm_sql:  SQL query error; rejecting user");
 		sql_release_socket(inst, sqlsocket);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
-		return RLM_MODULE_NOOP;
+		return RLM_MODULE_FAIL;
 
 	} else {
-
 		int     gcheck;
+		
+		radlog(L_DBG, "rlm_sql: User %s not found", sqlusername);
 
-		/*
-		 * We didn't find the user, so we try looking
-		 * for a DEFAULT entry
+                /*
+		 * We didn't find the user in radcheck, so we try looking
+		 * for radgroupcheck entry
 		 */
-		if (sql_set_user(inst, request, sqlusername, "DEFAULT") < 0) {
-			sql_release_socket(inst, sqlsocket);
-			return RLM_MODULE_FAIL;
-		}
-
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, NULL);
-		gcheck = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, NULL);
-		gcheck = sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
-		if (gcheck)
-			found = 1;
-	}
+                radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, sql_escape_func);
+                gcheck = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
+                radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, sql_escape_func);
+                sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
+                if (gcheck) {
+                        found = 1;
+                } else {
+                        /*
+                        * We didn't find the user, so we try looking
+                        * for a DEFAULT entry
+                        */
+                        if (sql_set_user(inst, request, sqlusername, "DEFAULT") < 0) {
+                                sql_release_socket(inst, sqlsocket);
+                                return RLM_MODULE_FAIL;
+                        }
+                        radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_check_query, request, sql_escape_func);
+                        gcheck = sql_getvpdata(inst, sqlsocket, &check_tmp, querystr, PW_VP_GROUPDATA);
+                        radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authorize_group_reply_query, request, sql_escape_func);
+                        gcheck = sql_getvpdata(inst, sqlsocket, &reply_tmp, querystr, PW_VP_GROUPDATA);
+                        if (gcheck)
+                                found = 1;
+                }
+        }
 	if (!found) {
-		radlog(L_DBG, "rlm_sql: User %s not found and DEFAULT not found", sqlusername);
+		radlog(L_DBG, "rlm_sql: DEFAULT not found");
 		sql_release_socket(inst, sqlsocket);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
@@ -357,7 +449,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	pairfree(&check_tmp);
 
 	if (inst->config->authenticate_query){
-		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authenticate_query, request, NULL);
+		radius_xlat(querystr, MAX_QUERY_LEN, inst->config->authenticate_query, request, sql_escape_func);
 	
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
@@ -411,7 +503,6 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	int     acctsessiontime = 0;
 #endif
 
-
 	memset(querystr, 0, MAX_QUERY_LEN);
 
 	/*
@@ -420,7 +511,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	if ((pair = pairfind(request->packet->vps, PW_ACCT_STATUS_TYPE)) != NULL) {
 		acctstatustype = pair->lvalue;
 	} else {
-		radius_xlat(logstr, MAX_QUERY_LEN, "rlm_sql:  packet has no account status type.  [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, NULL);
+		radius_xlat(logstr, MAX_QUERY_LEN, "rlm_sql:  packet has no account status type.  [user '%{User-Name}', nas '%{NAS-IP-Address}']", request, sql_escape_func);
 		radlog(L_ERR, logstr);
 		return RLM_MODULE_INVALID;
 	}
@@ -433,12 +524,12 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 		case PW_STATUS_ACCOUNTING_ON:
 		case PW_STATUS_ACCOUNTING_OFF:
 			radlog(L_INFO, "rlm_sql:  received Acct On/Off packet");
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_onoff_query, request, NULL);
+			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_onoff_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
-				return(RLM_MODULE_NOOP);
+				return(RLM_MODULE_FAIL);
 			if (querystr) {
 				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0)
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting for Acct On/Off packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
@@ -452,12 +543,17 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			 */
 		case PW_STATUS_ALIVE:
 
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_update_query, request, NULL);
+			/*
+			 * Set, escape, and check the user attr here
+			 */
+			sql_set_user(inst, request, sqlusername, NULL);
+
+			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_update_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
-				return(RLM_MODULE_NOOP);
+				return(RLM_MODULE_FAIL);
 			if (querystr) {
 				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0)
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting for ALIVE packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
@@ -474,15 +570,14 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			/*
 			 * Set, escape, and check the user attr here
 			 */
-			if(sql_set_user(inst, request, sqlusername, 0) < 0)
-				return RLM_MODULE_FAIL;
+			sql_set_user(inst, request, sqlusername, NULL);
 
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_start_query, request, NULL);
+			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_start_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
-				return(RLM_MODULE_NOOP);
+				return(RLM_MODULE_FAIL);
 			if (querystr) {
 				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0) {
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting" " for START packet - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
@@ -492,7 +587,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 					 * the stop record came before the start.  We try an
 					 * our alternate query now (typically an UPDATE)
 					 */
-					radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_start_query_alt, request, NULL);
+					radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_start_query_alt, request, sql_escape_func);
 					query_log(inst, querystr);
 
 					if (querystr) {
@@ -514,15 +609,14 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			/*
 			 * Set, escape, and check the user attr here
 			 */
-			if(sql_set_user(inst, request, sqlusername, 0) < 0)
-				return RLM_MODULE_FAIL;
+			sql_set_user(inst, request, sqlusername, NULL);
 
-			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_stop_query, request, NULL);
+			radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_stop_query, request, sql_escape_func);
 			query_log(inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
-				return(RLM_MODULE_NOOP);
+				return(RLM_MODULE_FAIL);
 			if (querystr) {
 				if ((inst->module->sql_query)(sqlsocket, inst->config, querystr) < 0) {
 					radlog(L_ERR, "rlm_sql: Couldn't update SQL accounting STOP record - %s", (char *)(inst->module->sql_error)(sqlsocket, inst->config));
@@ -547,14 +641,14 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 					                acctsessiontime = pair->lvalue;
 	
 					        if (acctsessiontime <= 0) {
-				        	        radius_xlat(logstr, MAX_QUERY_LEN, "rlm_sql:  Stop packet with zero session length.  (user '%{User-Name}', nas '%{NAS-IP-Address}')", request, NULL);
+				        	        radius_xlat(logstr, MAX_QUERY_LEN, "rlm_sql:  Stop packet with zero session length.  (user '%{User-Name}', nas '%{NAS-IP-Address}')", request, sql_escape_func);
 					                radlog(L_ERR, logstr);
 				        	        sql_release_socket(inst, sqlsocket);
 				                	return RLM_MODULE_NOOP;
 					        }
 #endif
 
-						radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_stop_query_alt, request, NULL);
+						radius_xlat(querystr, MAX_QUERY_LEN, inst->config->accounting_stop_query_alt, request, sql_escape_func);
 						query_log(inst, querystr);
 
 						if (querystr) {
@@ -576,6 +670,134 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 }
 
 
+/*
+ *        See if a user is already logged in. Sets request->simul_count to the
+ *        current session count for this user.
+ * 
+ *        Check twice. If on the first pass the user exceeds his
+ *        max. number of logins, do a second pass and validate all
+ *        logins by querying the terminal server (using eg. SNMP).
+ */
+
+static int rlm_sql_checksimul(void *instance, REQUEST * request) {
+	SQLSOCK 	*sqlsocket;
+	SQL_INST	*inst = instance;
+	SQL_ROW		row;
+	char		querystr[MAX_QUERY_LEN];
+	char		sqlusername[2*MAX_STRING_LEN+10];
+	int		check = 0;
+        uint32_t        ipno = 0;
+        char            *call_num = NULL;
+	VALUE_PAIR      *vp;
+
+	/* If simul_count_query is not defined, we don't do any checking */
+	if (inst->config->simul_count_query[0] == 0) {
+		return RLM_MODULE_NOOP;
+	}
+
+	if((request->username == NULL) || (request->username->length == 0)) {
+		radlog(L_ERR, "Zero Length username not permitted\n");
+		return RLM_MODULE_INVALID;
+	}
+
+	/* initialize the sql socket */
+	sqlsocket = sql_get_socket(inst);
+	if(sqlsocket == NULL)
+		return RLM_MODULE_FAIL;
+
+	if(sql_set_user(inst, request, sqlusername, 0) <0)
+		return RLM_MODULE_FAIL;
+
+	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->simul_count_query, request, NULL);
+	if((inst->module->sql_select_query)(sqlsocket, inst->config, querystr) < 0) {
+		radlog(L_ERR, "sql_checksimul: Database query failed");
+		sql_release_socket(inst, sqlsocket);
+		return RLM_MODULE_FAIL;
+	}
+
+	row = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
+	request->simul_count = atoi(row[0]);
+	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+
+	if(request->simul_count < request->simul_max) {
+		sql_release_socket(inst, sqlsocket);
+		return RLM_MODULE_OK;
+	}
+
+	/* Looks like too many sessions, so lets start verifying them */
+
+	if (inst->config->simul_verify_query[0] == 0) {
+		/* No verify query defined, so skip verify step and rely on count query only */
+		return RLM_MODULE_OK;
+	}
+
+	radius_xlat(querystr, MAX_QUERY_LEN, inst->config->simul_verify_query, request, NULL);
+	if((inst->module->sql_select_query)(sqlsocket, inst->config, querystr) < 0) {
+		radlog(L_ERR, "sql_checksimul: Database query error");
+		sql_release_socket(inst, sqlsocket);
+		return RLM_MODULE_FAIL;
+	}
+
+        /*
+         *      Setup some stuff, like for MPP detection.
+         */
+	request->simul_count = 0;
+
+        if ((vp = pairfind(request->packet->vps, PW_FRAMED_IP_ADDRESS)) != NULL)
+                ipno = vp->lvalue;
+        if ((vp = pairfind(request->packet->vps, PW_CALLING_STATION_ID)) != NULL)
+                call_num = vp->strvalue;        
+
+
+	while((row=(inst->module->sql_fetch_row)(sqlsocket, inst->config))) {
+		check = rad_check_ts((uint32_t)row[3], (int)row[4], row[2], row[1]);
+
+                /*
+                 *      Failed to check the terminal server for
+                 *      duplicate logins: Return an error.
+                 */
+		if (check < 0) {
+			return RLM_MODULE_FAIL;
+		}
+
+		if(check == 1) {
+			++request->simul_count;
+
+                        /*
+                         *      Does it look like a MPP attempt?
+                         */
+                        if (row[5] && ipno && inet_addr(row[5]) == ipno)
+                                request->simul_mpp = 2;
+                        else if (row[6] && call_num &&
+                                !strncmp(row[6],call_num,16))
+                                request->simul_mpp = 2;
+		}
+		else {
+                        /*
+                         *      Stale record - zap it.
+                         */
+			if(inst->config->simul_zap_query[0] != 0) {
+				SQLSOCK *sqlsocket1;
+				radlog(L_ERR, "rlm_sql: Deleting stale session [%s] (from %s/%s)", 
+								row[1],row[3],row[4]);
+				sqlsocket1 = sql_get_socket(inst);
+				sprintf(querystr, inst->config->simul_zap_query, row[0]);
+				(inst->module->sql_query)(sqlsocket1, inst->config, querystr);
+				(inst->module->sql_finish_select_query)(sqlsocket1, inst->config);
+				sql_release_socket(inst, sqlsocket1);
+			}
+		}
+	}
+
+	(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
+	sql_release_socket(inst, sqlsocket);
+
+	/* The Auth module apparently looks at request->simul_count, not the return value
+	   of this module when deciding to deny a call for too many sessions */
+	return RLM_MODULE_OK;
+
+}
+
 /* globally exported name */
 module_t rlm_sql = {
 	"SQL",
@@ -587,7 +809,7 @@ module_t rlm_sql = {
 		rlm_sql_authorize,	/* authorization */
 		NULL,			/* preaccounting */
 		rlm_sql_accounting,	/* accounting */
-		NULL			/* checksimul */
+		rlm_sql_checksimul	/* checksimul */
 	},
 	rlm_sql_detach,		/* detach */
 	rlm_sql_destroy,	/* destroy */
