@@ -71,6 +71,9 @@ typedef struct sql_inst {
 	SQL_CONFIG	*config;
 
 	rlm_sql_module_t *module;
+
+        int socknr;
+        pthread_mutex_t mutex;
 } SQL_INST;
 
 
@@ -87,6 +90,4 @@ int     sql_read_naslist(SQLSOCK * sqlsocket);
 int     sql_read_clients(SQLSOCK * sqlsocket);
 int     sql_dict_init(SQLSOCK * sqlsocket);
 void    query_log(SQL_INST * inst, char *querystr);
-int     sql_escape_string(char *to, char *from, int length);
-int     sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, char *username);
 #endif
