@@ -1,6 +1,6 @@
 /*
  * x99_mac.c
- * $Id: x99_mac.c,v 1.3 2002/01/17 04:02:17 fcusack Exp $
+ * $Id: x99_mac.c,v 1.5 2002/05/30 04:21:10 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,14 +19,16 @@
  * Copyright 2001,2002  Google, Inc.
  */
 
+#ifdef FREERADIUS
 #include "autoconf.h"
 #include "radiusd.h"
+#endif
 #include "x99.h"
 
 #include <string.h>
 #include <openssl/des.h>
 
-static const char rcsid[] = "$Id: x99_mac.c,v 1.3 2002/01/17 04:02:17 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_mac.c,v 1.5 2002/05/30 04:21:10 fcusack Exp $";
 
 
 /*
@@ -63,7 +65,7 @@ x99_response(const char *challenge, char response[9],
 	    conversion = x99_cc_dec_conversion;
 	} else {
 	    /* This should not happen. */
-	    radlog(L_ERR, "rlm_x99_token: x99_response: bad card mode/vendor");
+	    x99_log(X99_LOG_ERR, "x99_response: bad card mode/vendor");
 	    return -1;
 	}
     } else {
@@ -79,7 +81,7 @@ x99_response(const char *challenge, char response[9],
 	    (void) memmove(&response[3], &response[4], 5);
 	} else {
 	    /* This should not happen. */
-	    radlog(L_ERR, "rlm_x99_token: x99_response: bad card mode/vendor");
+	    x99_log(X99_LOG_ERR, "x99_response: bad card mode/vendor");
 	    return -1;
 	}
     }
@@ -121,8 +123,8 @@ x99_mac(const char *input, des_cblock output, des_cblock keyblock)
      * actually profiled it.
      */
     if ((rc = des_set_key_checked((const_des_cblock *) keyblock, ks)) != 0) {
-	radlog(L_ERR, "rlm_x99_token: x99_mac: DES key %s",
-	       rc == -1 ? "has incorrect parity" : "is weak");
+	x99_log(X99_LOG_ERR, "x99_mac: DES key %s",
+		rc == -1 ? "has incorrect parity" : "is weak");
 	return -1;
     }
 
