@@ -1,6 +1,6 @@
 /*
  * x99_pwe.c
- * $Id: x99_pwe.c,v 1.9 2002/01/19 01:20:24 fcusack Exp $
+ * $Id: x99_pwe.c,v 1.14 2002/06/24 10:13:58 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -26,9 +26,11 @@
  * interface is not sufficient for x9.9 use.
  */
 
+#ifdef FREERADIUS
 #include "autoconf.h"
 #include "libradius.h"
 #include "rad_assert.h"
+#endif
 #include "x99.h"
 #include "x99_pwe.h"
 
@@ -37,7 +39,9 @@
 #include <openssl/md5.h>
 #include <openssl/sha.h>
 
-static const char rcsid[] = "$Id: x99_pwe.c,v 1.9 2002/01/19 01:20:24 fcusack Exp $";
+#include <string.h>
+
+static const char rcsid[] = "$Id: x99_pwe.c,v 1.14 2002/06/24 10:13:58 fcusack Exp $";
 
 
 /* Attribute IDs for supported password encodings. */
@@ -57,7 +61,7 @@ x99_pwe_init(void)
      *     It should be sized as (number of password types * 2)
      * This is temporary until this code becomes modular.
      */
-    memset(pwattr, 0, sizeof(pwattr));
+    (void) memset(pwattr, 0, sizeof(pwattr));
 
     /* PAP */
     if ((da = dict_attrbyname("User-Password")) != NULL) {
@@ -178,7 +182,7 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 	    break;
 	}
 	if (resp_vp->length != 17) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: CHAP-Password wrong size");
+	    x99_log(X99_LOG_AUTH, "pw_valid: CHAP-Password wrong size");
 	    match = 0;
 	    break;
 	}
@@ -218,27 +222,24 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 
 	DEBUG("rlm_x99_token: pw_valid: handling PW_MS_CHAP_RESPONSE");
 	if (chal_vp->length != 8) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAP-Challenge wrong size");
+	    x99_log(X99_LOG_AUTH, "pw_valid: MS-CHAP-Challenge wrong size");
 	    match = 0;
 	    break;
 	}
 	if (resp_vp->length != 50) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAP-Response wrong size");
+	    x99_log(X99_LOG_AUTH, "pw_valid: MS-CHAP-Response wrong size");
 	    match = 0;
 	    break;
 	}
 	if ((resp_vp->strvalue)[1] != 1) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAP-Response wrong flags (LM not supported)");
+	    x99_log(X99_LOG_AUTH,
+		    "pw_valid: MS-CHAP-Response bad flags (LM not supported)");
 	    match = 0;
 	    break;
 	}
 	/* This is probably overkill. */
 	if (strlen(password) > MAX_STRING_LEN) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAP password too long");
+	    x99_log(X99_LOG_AUTH, "pw_valid: MS-CHAP password too long");
 	    match = 0;
 	    break;
 	}
@@ -363,21 +364,18 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 
 	DEBUG("rlm_x99_token: pw_valid: handling PW_MS_CHAP2_RESPONSE");
 	if (chal_vp->length != 16) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAP-Challenge (v2) wrong size");
+	    x99_log(X99_LOG_AUTH,"pw_valid: MS-CHAP-Challenge (v2) wrong size");
 	    match = 0;
 	    break;
 	}
 	if (resp_vp->length != 50) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAP2-Response wrong size");
+	    x99_log(X99_LOG_AUTH, "pw_valid: MS-CHAP2-Response wrong size");
 	    match = 0;
 	    break;
 	}
 	/* This is probably overkill. */
 	if (strlen(password) > MAX_STRING_LEN) {
-	    radlog(L_AUTH, "rlm_x99_token: pw_valid: "
-			   "MS-CHAPv2 password too long");
+	    x99_log(X99_LOG_AUTH, "pw_valid: MS-CHAPv2 password too long");
 	    match = 0;
 	    break;
 	}
@@ -592,6 +590,7 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 
 	    SHA_CTX ctx;
 	    unsigned char sha_md[SHA_DIGEST_LENGTH];
+#if 0 /* salting/encoding now handled in lib/radius.c:tunnel_pwencode() */
 	    unsigned char md5_md[MD5_DIGEST_LENGTH];
 
 	    /*   From RFC 2548:           S                 R           A */
@@ -605,9 +604,10 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 	    unsigned char mppe_key_string[2 + (2 * sizeof(salt)) +
 	    /*				  (   ASCII(mppe_key)  )  \0 */
 					  (2 * sizeof(mppe_key)) + 1];
-
-	    (void) memset(mppe_key, 0, 32);
-	    mppe_key[0] = 16; /* length (s/rant//) */
+#else /* 0 */
+	    /*                           0x   (   ASCII(mppe_key)   )  \0 */
+	    unsigned char mppe_key_string[2 + (2 * sizeof(MasterKey)) + 1];
+#endif /* 0 */
 
 	    /* Generate the master session key. */
 	    SHA1_Init(&ctx);
@@ -637,11 +637,14 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 
 	    /* Now, generate the MS-MPPE-Send-Key attribute. */
 
+#if 0
 	    /* Setup the salt value. */
 	    salt[0] = 0x80;
 	    salt[1] = 0x01;
 
 	    /* Encode the key. */
+	    (void) memset(mppe_key, 0, sizeof(mppe_key));
+	    mppe_key[0] = 16; /* length */
 	    (void) memcpy(&mppe_key[1], MasterSendKey, 16);
 	    secretlen = strlen(request->secret);
 	    (void) memcpy(encode_buf, request->secret, secretlen);
@@ -663,17 +666,27 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 	    (void) sprintf(&mppe_key_string[4], "%02X", salt[1]);
 	    for (i = 0; i < sizeof(mppe_key); ++i)
 		(void) sprintf(&mppe_key_string[i*2+6], "%02X", mppe_key[i]);
+#else /* 0 */
+	    mppe_key_string[0] = '0';
+	    mppe_key_string[1] = 'x';
+	    for (i = 0; i < sizeof(MasterSendKey); ++i)
+		(void) sprintf(&mppe_key_string[i*2+2], "%02X",
+			       MasterSendKey[i]);
+#endif /* 0 */
 	    vp = pairmake("MS-MPPE-Send-Key", mppe_key_string, T_OP_EQ);
 	    rad_assert(vp != NULL);
 	    pairadd(vps, vp);
 
 	    /* Generate the MS-MPPE-Recv-Key attribute. */
 
+#if 0
 	    /* Setup the salt value. */
 	    salt[0] = 0x80;
 	    salt[1] = 0x02;
 
 	    /* Encode the key. */
+	    (void) memset(mppe_key, 0, sizeof(mppe_key));
+	    mppe_key[0] = 16; /* length */
 	    (void) memcpy(&mppe_key[1], MasterReceiveKey, 16);
 	    secretlen = strlen(request->secret);
 	    (void) memcpy(encode_buf, request->secret, secretlen);
@@ -695,6 +708,13 @@ x99_pw_valid(const REQUEST *request, x99_token_t *inst,
 	    (void) sprintf(&mppe_key_string[4], "%02X", salt[1]);
 	    for (i = 0; i < sizeof(mppe_key); ++i)
 		(void) sprintf(&mppe_key_string[i*2+6], "%02X", mppe_key[i]);
+#else /* 0 */
+	    mppe_key_string[0] = '0';
+	    mppe_key_string[1] = 'x';
+	    for (i = 0; i < sizeof(MasterReceiveKey); ++i)
+		(void) sprintf(&mppe_key_string[i*2+2], "%02X",
+			       MasterReceiveKey[i]);
+#endif /* 0 */
 	    vp = pairmake("MS-MPPE-Recv-Key", mppe_key_string, T_OP_EQ);
 	    rad_assert(vp != NULL);
 	    pairadd(vps, vp);
