@@ -1,6 +1,6 @@
 /*
  * x99_state.c
- * $Id: x99_state.c,v 1.5 2002/01/17 04:02:18 fcusack Exp $
+ * $Id: x99_state.c,v 1.7 2002/05/30 04:21:11 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,9 +19,11 @@
  * Copyright 2001,2002  Google, Inc.
  */
 
+#ifdef FREERADIUS
 #include "autoconf.h"
 #include "libradius.h"
 #include "radiusd.h"
+#endif
 #include "x99.h"
 
 #include <string.h>
@@ -29,7 +31,7 @@
 #include <openssl/hmac.h>
 
 
-static const char rcsid[] = "$Id: x99_state.c,v 1.5 2002/01/17 04:02:18 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_state.c,v 1.7 2002/05/30 04:21:11 fcusack Exp $";
 
 
 /*
