@@ -1,6 +1,6 @@
 /*
  * x99_sync.c
- * $Id: x99_sync.c,v 1.4 2002/01/18 22:47:08 fcusack Exp $
+ * $Id: x99_sync.c,v 1.9 2002/07/03 09:03:47 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,9 +19,11 @@
  * Copyright 2001,2002  Google, Inc.
  */
 
+#ifdef FREERADIUS
 #include "autoconf.h"
 #include "libradius.h"
 #include "radiusd.h"
+#endif
 #include "x99.h"
 #include "x99_sync.h"
 
@@ -35,7 +37,7 @@
 #include <unistd.h>
 #include <openssl/des.h> /* des_cblock */
 
-static const char rcsid[] = "$Id: x99_sync.c,v 1.4 2002/01/18 22:47:08 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_sync.c,v 1.9 2002/07/03 09:03:47 fcusack Exp $";
 
 
 /*
@@ -177,23 +179,6 @@ x99_upd_last_auth(const char *syncdir, const char *username)
 }
 
 
-/*
- * Return the failed login count and last auth time for a user.
- * Returns 0 on success, non-zero otherwise.
- */
-int
-x99_get_failcount(const char *syncdir, const char *username, int *failcount)
-{
-    int rc;
-    char *lock;
-
-    if ((lock = x99_acquire_sd_lock(syncdir, username)) == NULL)
-	return -1;
-    rc = x99_get_sd(syncdir, username, NULL, failcount, NULL);
-    x99_release_sd_lock(lock);
-    return rc;
-}
-
 /*
  * Atomically increment a user's failed login count.
  * Also updates last_auth.
@@ -248,6 +233,98 @@ x99_reset_failcount(const char *syncdir, const char *username)
 }
 
 
+/*
+ * checks the failure counter.
+ * returns 0 if the user is allowed to authenticate, -1 otherwise.
+ * caller does not need to log failures, we do it.
+ */
+int
+x99_check_failcount(const char *username, const x99_token_t *inst)
+{
+    time_t last_auth;
+    int failcount;
+
+    if (x99_get_last_auth(inst->syncdir, username, &last_auth) != 0) {
+	x99_log(X99_LOG_ERR,
+		"auth: unable to get last auth time for [%s]", username);
+	return -1;
+    }
+    if (x99_get_failcount(inst->syncdir, username, &failcount) != 0) {
+	x99_log(X99_LOG_ERR,
+		"auth: unable to get failure count for [%s]", username);
+	return -1;
+    }
+
+    /* Check against hardfail setting. */
+    if (inst->hardfail && failcount >= inst->hardfail) {
+	x99_log(X99_LOG_AUTH,
+		"auth: %d/%d failed/max authentications for [%s]",
+		failcount, inst->hardfail, username);
+	if (x99_incr_failcount(inst->syncdir, username) != 0) {
+	    x99_log(X99_LOG_ERR,
+		    "auth: unable to increment failure count for "
+		    "locked out user [%s]", username);
+	}
+	return -1;
+    }
+
+    /* Check against softfail setting. */
+    if (failcount >= inst->softfail) {
+	time_t when;
+	int fcount;
+
+	/*
+	 * Determine the next time this user can authenticate.
+	 *
+	 * Once we hit softfail, we introduce a 1m delay before the user
+	 * can authenticate.  For each successive failed authentication,
+	 * we double the delay time, up to a max of 32 minutes.  While in
+	 * the "delay mode" of operation, all authentication ATTEMPTS are
+	 * considered failures (we don't test if the password is correct).
+	 * Also, each attempt during the delay period restarts the clock.
+	 *
+	 * The advantage of a delay instead of a simple lockout is that an
+	 * attacker can't lock out a user as easily; the user need only wait
+	 * a bit before he can authenticate.
+	 */
+	fcount = failcount - inst->softfail;
+	when = last_auth + (fcount > 5 ? 32 * 60 : (1 << fcount) * 60);
+	if (time(NULL) < when) {
+	    x99_log(X99_LOG_AUTH,
+		    "auth: user [%s] auth too soon while delayed, "
+		    "%d/%d failed/softfail authentications",
+		    username, failcount, inst->softfail);
+	    if (x99_incr_failcount(inst->syncdir, username) != 0) {
+		x99_log(X99_LOG_ERR,
+			"auth: unable to increment failure count for "
+			"delayed user [%s]", username);
+	    }
+	    return -1;
+	}
+    }
+
+    return 0;
+}
+
+
+/*
+ * Return the failed login count and last auth time for a user.
+ * Returns 0 on success, non-zero otherwise.
+ */
+static int
+x99_get_failcount(const char *syncdir, const char *username, int *failcount)
+{
+    int rc;
+    char *lock;
+
+    if ((lock = x99_acquire_sd_lock(syncdir, username)) == NULL)
+	return -1;
+    rc = x99_get_sd(syncdir, username, NULL, failcount, NULL);
+    x99_release_sd_lock(lock);
+    return rc;
+}
+
+
 /*
  * Sync data is kept in a flat file[s], only because it's easy to implement.
  * It might be worth looking at Berkeley DB, but the flat file implementation
@@ -280,18 +357,17 @@ static char *
 x99_acquire_sd_lock(const char *syncdir, const char *username)
 {
     char *lockfile;
-    int i, fd;
+    int i, fd = -1;
     struct stat st;
 
     /* Verify permissions first. */
     if (stat(syncdir, &st) != 0) {
-	radlog(L_ERR, "rlm_x99_token: syncdir %s error: %s",
-	       syncdir, strerror(errno));
+	x99_log(X99_LOG_ERR, "syncdir %s error: %s",
+		syncdir, strerror(errno));
 	return NULL;
     }
     if (st.st_mode != (S_IFDIR|S_IRUSR|S_IWUSR|S_IXUSR)) {
-	radlog(L_ERR, "rlm_x99_token: syncdir %s has loose permissions",
-	       syncdir);
+	x99_log(X99_LOG_ERR, "syncdir %s has loose permissions", syncdir);
 	return NULL;
     }
 
@@ -312,9 +388,10 @@ x99_acquire_sd_lock(const char *syncdir, const char *username)
 	/* FIXME: does usleep() reset/generate SIGALRM on any systems? */
 	usleep(500000); /* 0.5 second */
     }
-    if (fd  == -1) {
-	radlog(L_ERR, "x99_rlm_token: x99_acquire_sd_lock: unable to acquire "
-		      "lock for [%s]", username);
+    if (fd == -1) {
+	x99_log(X99_LOG_ERR,
+		"x99_acquire_sd_lock: unable to acquire lock for [%s]",
+		username);
 	free(lockfile);
 	return NULL;
     }
@@ -356,9 +433,8 @@ x99_get_sd(const char *syncdir, const char *username,
     /* Open sync file. */
     if ((fp = fopen(syncfile, "r")) == NULL) {
 	if (errno != ENOENT) {
-	    radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-			  "unable to open sync file %s: %s",
-			  syncfile, strerror(errno));
+	    x99_log(X99_LOG_ERR, "x99_get_sd: unable to open sync file %s: %s",
+		    syncfile, strerror(errno));
 	    return -1;
 	}
 	/*
@@ -372,9 +448,8 @@ x99_get_sd(const char *syncdir, const char *username,
 
     /* Read sync data. */
     if ((fgets(syncdata, sizeof(syncdata), fp) == NULL) || !strlen(syncdata)) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "unable to read sync data from %s: %s",
-		       syncfile, strerror(errno));
+	x99_log(X99_LOG_ERR, "x99_get_sd: unable to read sync data from %s: %s",
+		syncfile, strerror(errno));
 	(void) fclose(fp);
 	return -1;
     }
@@ -384,37 +459,38 @@ x99_get_sd(const char *syncdir, const char *username,
     /* Now, parse the sync data. */
     /* Eat the version. */
     if ((p = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data for user %s", username);
 	return -1;
     }
     p++;
 
     /* Sanity check the username. */
     if ((q = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data for user %s", username);
 	return -1;
     }
     *q++ = '\0';
     if (strcmp(p, username)) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "sync data user mismatch for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: sync data user mismatch for user %s", username);
 	return -1;
     }
     p = q;
 
     /* Get challenge. */
     if ((q = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (challenge) for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (challenge) for user %s",
+		username);
 	return -1;
     }
     *q++ = '\0';
     if (strlen(p) > MAX_CHALLENGE_LEN) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (challenge length) for user %s",
-		      username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (challenge length) for user %s",
+		username);
 	return -1;
     }
     if (challenge)
@@ -423,36 +499,40 @@ x99_get_sd(const char *syncdir, const char *username,
 
     /* Eat key. */
     if ((p = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (key) for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (key) for user %s", username);
 	return -1;
     }
     p++;
 
     /* Get failures. */
     if ((q = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (failures) for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (failures) for user %s",
+		username);
 	return -1;
     }
     *q++ = '\0';
     if (failures && (sscanf(p, "%d", failures) != 1)) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (failures) for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (failures) for user %s",
+		username);
 	return -1;
     }
     p = q;
 
     /* Get last_auth. */
     if ((q = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (last_auth) for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (last_auth) for user %s",
+		username);
 	return -1;
     }
     *q++ = '\0';
     if (last_auth && (sscanf(p, "%ld", last_auth) != 1)) {
-	radlog(L_ERR, "rlm_x99_token: x99_get_sd: "
-		      "invalid sync data (last_auth) for user %s", username);
+	x99_log(X99_LOG_ERR,
+		"x99_get_sd: invalid sync data (last_auth) for user %s",
+		username);
 	return -1;
     }
 
@@ -475,9 +555,8 @@ x99_set_sd(const char *syncdir, const char *username,
     syncfile[PATH_MAX] = '\0';
 
     if ((fp = fopen(syncfile, "w")) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: x99_set_sd: "
-		      "unable to open sync file %s: %s",
-		      syncfile, strerror(errno));
+	x99_log(X99_LOG_ERR, "x99_set_sd: unable to open sync file %s: %s",
+		syncfile, strerror(errno));
 	return -1;
     }
 
@@ -485,9 +564,8 @@ x99_set_sd(const char *syncdir, const char *username,
     (void) fprintf(fp, "1:%s:%s:%s:%d:%ld:\n", username, challenge, "",
 		   failures, last_auth);
     if (fclose(fp) != 0) {
-	radlog(L_ERR, "rlm_x99_token: x99_set_sd: "
-		      "unable to write sync file %s: %s",
-		      syncfile, strerror(errno));
+	x99_log(X99_LOG_ERR, "x99_set_sd: unable to write sync file %s: %s",
+		syncfile, strerror(errno));
 	return -1;
     }
 
