@@ -1,6 +1,6 @@
 /*
  * x99_util.c	
- * $Id: x99_util.c,v 1.4 2002/01/17 04:02:18 fcusack Exp $
+ * $Id: x99_util.c,v 1.6 2002/05/30 04:21:11 fcusack Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,8 +19,10 @@
  * Copyright 2001,2002  Google, Inc.
  */
 
+#ifdef FREERADIUS
 #include "autoconf.h"
 #include "radiusd.h"
+#endif
 #include "x99.h"
 
 #include <stdlib.h>
@@ -33,7 +35,7 @@
 #include <openssl/des.h> /* des_cblock */
 
 
-static const char rcsid[] = "$Id: x99_util.c,v 1.4 2002/01/17 04:02:18 fcusack Exp $";
+static const char rcsid[] = "$Id: x99_util.c,v 1.6 2002/05/30 04:21:11 fcusack Exp $";
 
 
 /* Card name to int mappings */
@@ -61,6 +63,38 @@ static struct {
 };
 
 
+/*
+ * Return a random challenge.
+ * fd must be either -1 or an open fd to the random device.
+ * challenge is filled in on successful return (must be size len+1).
+ * Returns 0 on success, -1 on failure.
+ */
+int
+x99_get_challenge(int fd, char *challenge, int len)
+{
+    unsigned char rawchallenge[MAX_CHALLENGE_LEN];
+    int i;
+
+    if (fd == -1) {
+	if ((fd = open(DEVURANDOM, O_RDONLY)) == -1) {
+	    x99_log(X99_LOG_ERR, "error opening %s: %s", DEVURANDOM,
+		    strerror(errno));
+	    return -1;
+	}
+    }
+
+    if (x99_get_random(fd, rawchallenge, len) == -1) {
+	x99_log(X99_LOG_ERR, "failed to obtain random data");
+	return -1;
+    }
+    /* Convert the raw bytes to a decimal string. */
+    for (i = 0; i < len; ++i)
+	challenge[i] = '0' + rawchallenge[i] % 10;
+    challenge[i] = '\0';
+
+    return 0;
+}
+
 /*
  * Return some number of random bytes.
  * rnd_data must be allocated by the caller.
@@ -76,8 +110,8 @@ x99_get_random(int fd, unsigned char *rnd_data, int req_bytes)
 
 	n = read(fd, rnd_data + bytes_read, req_bytes - bytes_read);
 	if (n <= 0) {
-	    radlog(L_ERR, "rlm_x99_token: error reading from %s: %s",
-		   DEVURANDOM, strerror(errno));
+	    x99_log(X99_LOG_ERR, "x99_get_random: error reading from %s: %s",
+		    DEVURANDOM, strerror(errno));
 	    return -1;
 	}
 	bytes_read += n;
@@ -123,9 +157,9 @@ x99_string_to_keyblock(const char *s, des_cblock keyblock)
 
 
 /* Character maps for generic hex and vendor specific decimal modes */
-const char x99_hex_conversion[]   = "0123456789abcdef";
-const char x99_cc_dec_conversion[]  = "0123456789012345";
-const char x99_snk_dec_conversion[] = "0123456789222333";
+const char x99_hex_conversion[]         = "0123456789abcdef";
+const char x99_cc_dec_conversion[]      = "0123456789012345";
+const char x99_snk_dec_conversion[]     = "0123456789222333";
 const char x99_sc_friendly_conversion[] = "0123456789ahcpef";
 
 /*
@@ -168,19 +202,19 @@ x99_get_user_info(const char *pwdfile, const char *username,
 
     /* Verify permissions first. */
     if (stat(pwdfile, &st) != 0) {
-	radlog(L_ERR, "rlm_x99_token: pwdfile %s error: %s",
-	       pwdfile, strerror(errno));
+	x99_log(X99_LOG_ERR, "x99_get_user_info: pwdfile %s error: %s",
+		pwdfile, strerror(errno));
 	return -2;
     }
     if ((st.st_mode & (S_IXUSR|S_IRWXG|S_IRWXO)) != 0) {
-	radlog(L_ERR, "rlm_x99_token: pwdfile %s has loose permissions",
-	       pwdfile);
+	x99_log(X99_LOG_ERR,
+		"x99_get_user_info: pwdfile %s has loose permissions", pwdfile);
 	return -2;
     }
 
     if ((fp = fopen(pwdfile, "r")) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: error opening %s: %s",
-	       pwdfile, strerror(errno));
+	x99_log(X99_LOG_ERR, "x99_get_user_info: error opening %s: %s",
+		pwdfile, strerror(errno));
 	return -2;
     }
 
@@ -194,8 +228,9 @@ x99_get_user_info(const char *pwdfile, const char *username,
     while (!feof(fp)) {
 	if (fgets(s, sizeof(s), fp) == NULL) {
 	    if (!feof(fp)) {
-		radlog(L_ERR, "rlm_x99_token: error reading from %s: %s",
-		       pwdfile, strerror(errno));
+		x99_log(X99_LOG_ERR,
+			"x99_get_user_info: error reading from %s: %s",
+			pwdfile, strerror(errno));
 		(void) fclose(fp);
 		free(p);
 		return -2;
@@ -208,22 +243,24 @@ x99_get_user_info(const char *pwdfile, const char *username,
     (void) fclose(fp);
     free(p);
     if (!found) {
-	radlog(L_AUTH, "rlm_x99_token: [%s] not found in %s",
-	       username, pwdfile);
+	x99_log(X99_LOG_AUTH, "x99_get_user_info: [%s] not found in %s",
+		username, pwdfile);
 	return -1;
     }
 
     /* Found him, skip to next field (card). */
     if ((p = strchr(s, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: invalid format for [%s] in %s",
-	       username, pwdfile);
+	x99_log(X99_LOG_ERR,
+		"x99_get_user_info: invalid format for [%s] in %s",
+		username, pwdfile);
 	return -2;
     }
     p++;
     /* strtok() */
     if ((q = strchr(p, ':')) == NULL) {
-	radlog(L_ERR, "rlm_x99_token: invalid format for [%s] in %s",
-	       username, pwdfile);
+	x99_log(X99_LOG_ERR,
+		"x99_get_user_info: invalid format for [%s] in %s",
+		username, pwdfile);
 	return -2;
     }
     *q++ = '\0';
@@ -239,15 +276,16 @@ x99_get_user_info(const char *pwdfile, const char *username,
 	}
     }
     if (!found) {
-	radlog(L_ERR, "rlm_x99_token: unknown card %s for [%s] in %s",
-	       p, username, pwdfile);
+	x99_log(X99_LOG_ERR,
+		"x99_get_user_info: unknown card %s for [%s] in %s",
+		p, username, pwdfile);
 	return -2;
     }
 
     if (!(strlen(q) == 16 || (strlen(q) == 17 && q[16] == '\n'))) {
 	/* 8 octets + possible trailing newline */
-	radlog(L_ERR, "rlm_x99_token: invalid key for [%s] in %s",
-	       username, pwdfile);
+	x99_log(X99_LOG_ERR, "x99_get_user_info: invalid key for [%s] in %s",
+		username, pwdfile);
 	return -2;
     }
 
